;************************************************************************************
;
; Microbot ROM bit correct disassembly - Lee Davison 09/2011

; 1K RAM from $0000 to $03FF
; echoed from $2000 to $23FF

LAB_00	= $00			; Rx buffer to $7F

LAB_80	= $80			; mode byte
					; value	mode
					; =====	====
					;  $FF	MOVE
					;  $F1	TRAIN
					;  $01	moving
					;  $00	done

LAB_81	= $81			; last button pressed
LAB_82	= $82			; @READ last button pressed
LAB_83	= $83			; last held button

LAB_84	= $84			; numeric value low byte
LAB_85	= $85			; numeric value high byte

LAB_86	= $86			; line received flag

LAB_87	= $87			; recognition character received flag
LAB_88	= $88			; recognition character, usually "@"

LAB_89	= $89			; button scan wait counter
LAB_8A	= $8A			; button scan wait count

LAB_8B	= $8B			; Rx buffer write index
LAB_8C	= $8C			; Rx buffer read index

LAB_8D	= $8D			; program step pointer low byte
LAB_8E	= $8E			; program step pointer high byte

LAB_8F	= $8F			; maximum delta low byte
LAB_90	= $90			; maximum delta high byte

LAB_91	= $91			; move step count low byte
LAB_92	= $92			; move step count high byte

LAB_94	= $94			; pause count low byte, program step number
LAB_95	= $95			; pause count high byte

LAB_96	= $96			; steps beyond the gripper switch point

LAB_97	= $97			; signed decimal input low byte, output word low byte,
					; data selector byte
LAB_98	= $98			; signed decimal input high byte, output word high byte
LAB_99	= $99			; signed decimal input negative flag

LAB_9A	= $9A			; @seT mode flag

LAB_9B	= $9B			; stepper index, word count, command index

LAB_9C	= $9C			; digit received flag, digit output flag
LAB_9D	= $9D			; digit value low byte
LAB_9E	= $9E			; digit value high byte

LAB_9F	= $9F			; no increment flag
					; if this byte is non zero the next command will not start
					; with an increment of the command pointer

LAB_A0	= $A0			; RAM end low byte
LAB_A1	= $A1			; RAM end high byte

LAB_A2	= $A2			; Tx character delay

LAB_A3	= $A3			; pendant column 5 state

LAB_A4	= $A4			; acceleration control flag
					; if non zero acceleration control is used
LAB_A5	= $A5			; T1Cl value
LAB_A6	= $A6			; T1Ch value
LAB_A8	= $A8			; timer 1 minimum value high byte
LAB_A9	= $A9			; acceleration step count

LAB_AA	= $AA			; command sequence error flag
					; this byte will be non zero if an ACIA A error is detected
					; after the recognition character has been received but
					; before the terminating [CR]

LAB_AC	= $AC			; SPEED value

LAB_AD	= $AD			; stepper x count low byte
LAB_B3	= $B3			; stepper x count high byte

LAB_B9	= $B9			; stepper x delta low byte
LAB_BB	= $BB			; elbow stepper delta low byte
LAB_BC	= $BC			; right wrist stepper delta low byte
LAB_BD	= $BD			; left wrist stepper delta low byte
LAB_BE	= $BE			; gripper stepper delta low byte

LAB_BF	= $BF			; stepper x delta high byte
LAB_C4	= $C4			; gripper stepper delta high byte

LAB_C5	= $C5			; stepper x direction low byte
LAB_CB	= $CB			; stepper x direction high byte

LAB_D1	= $D1			; interpolation total low byte
LAB_D7	= $D7			; interpolation total high byte

LAB_DD	= $DD			; stepper x phase offset

LAB_E3	= $E3			; stepper 0 drive pattern base
LAB_E4	= $E4			; stepper 1 drive pattern shoulder
LAB_E5	= $E5			; stepper 2 drive pattern elbow
LAB_E6	= $E6			; stepper 3 drive pattern right wrist
LAB_E7	= $E7			; stepper 4 drive pattern left wrist
LAB_E8	= $E8			; stepper 5 drive pattern gripper

LAB_E9	= $E9			; end of RAM to clear

; $E9 to $FF unused

LAB_0119	= $0119		; top of stack
LAB_011A	= $011A		; start of program step RAM

; 2K RAM from $0400 to $07FF
; echoed from $2400 to $27FF

LAB_0400	= $0400		; RAM end for 1K RAM
					; room for 53 $0E size program steps

LAB_07FE	= $07FE		; RAM end for 2K RAM
					; room for 126 $0E size program steps

LAB_07FF	= $07FF		; test address for 2K RAM detection


;************************************************************************************
;
; 74LS174 hex D-type flip flops

; these are probably wired with 4 bits all going to d3-d0 and a pair of bits from flip
; flops 0 and 2 going to two of d7-d4 and a pair of bits from flip flops 1 and 3 going
; to the other two of d7-d4

; echoed from $0800 to $0BFF
;         and $2800 to $2BFF

LAB_0800	= $0800		; 74LS174 hex D-type flip flop 0
					; stepper 0 and half of stepper 1 driver
					; U12, base and half of the shoulder

; echoed from $0C00 to $0FFF
;         and $2C00 to $2FFF

LAB_0C00	= $0C00		; 74LS174 hex D-type flip flop 1
					; stepper 2 and half of stepper 1 driver
					; U05, elbow and half of the shoulder

; echoed from $1000 to $13FF
;         and $3000 to $33FF

LAB_1000	= $1000		; 74LS174 hex D-type flip flop 2
					; stepper 3 and half of stepper 4 driver
					; U09, right wrist and half of the left wrist

; echoed from $1400 to $17FF
;         and $3400 to $37FF

LAB_1400	= $1400		; 74LS174 hex D-type flip flop 3
					; stepper 5 and half of stepper 4 driver
					; U02, gripper and half of the left wrist


;************************************************************************************
;
; 74LS251 8 input data selector - do not read from these addresses

; echoed from $1800 to $1BFF
;         and $3800 to $3BFF

; bit n is addressed at LAB_180n and goes to the CPU d7

LAB_1800	= $1800		; pin 12 P15 Pendant and P16 Gripper SW
;LAB_1801	= LAB_1800+$01	; pin 04 P17 User Optional Input
;LAB_1802	= LAB_1800+$02	; pin 06 P17 User Optional Input
;LAB_1803	= LAB_1800+$03	; pin 08 P17 User Optional Input
;LAB_1804	= LAB_1800+$04	; pin 10 P17 User Optional Input
;LAB_1805	= LAB_1800+$05	; pin 12 P17 User Optional Input
;LAB_1806	= LAB_1800+$06	; pin 14 P17 User Optional Input
;LAB_1807	= LAB_1800+$07	; pin 16 P17 User Optional Input


;************************************************************************************
;
; 74LS259 8 bit addressable latch, echoed to $1FFF

; echoed from $1C00 to $1FFF
;         and $3C00 to $3FFF

; bit n is addressed at LAB_1C0n and comes from the CPU d0

LAB_1C00	= $1C00		; pin 14 of P15 pendant, MODE LED
;LAB_1C01	= LAB_1C00+$01	; pin 13 of P17 user output
;LAB_1C02	= LAB_1C00+$02	; pin 11 of P17 user output
;LAB_1C03	= LAB_1C00+$03	; pin 09 of P17 user output
;LAB_1C04	= LAB_1C00+$04	; pin 07 of P17 user output
;LAB_1C05	= LAB_1C00+$05	; pin 05 of P17 user output
LAB_1C06	= LAB_1C00+$06	; pin 09 of P15 pendant, TRAIN LED
LAB_1C07	= LAB_1C00+$07	; pin 13 of P15 pendant, RUN LED


;************************************************************************************
;
; all of the address map from $0000 to $1FFF is echoed in the following 8K block so,
; for example, the 74LS251, 8 input data selector, also appears at $3800 to $3BFF


;************************************************************************************
;
; MC6850 asynchronous communications interface adapters

; echoed from $4000 to $4FFF
;         and $6000 to $6FFF

LAB_4800	= $4800		; ACIA A status port
LAB_4801	= LAB_4800+$01	; ACIA A data port

; echoed from $4000 to $47FF
;         and $5000 to $57FF
;         and $6000 to $67FF
;         and $7000 to $77FF

LAB_5000	= $5000		; ACIA B status port
LAB_5001	= LAB_5000+$01	; ACIA B data port


;************************************************************************************
;
; R6522 versatile interface adapter

; echoed from $2000 to $3FFF
;         and $6000 to $7FFF

LAB_7800	= $7800		; VIA DRB
					; bit	function
					; ---	--------
					;  7	unused
					;  6	ENTER LED
					;  5	pin 11 P15 pendant, button column 5
					;  4	pin 10 P15 pendant, button column 4
					;  3	sw 3
					;  2	baud rate sw 2
					;  1	baud rate sw 1
					;  0	baud rate sw 0
LAB_7801	= LAB_7800+$01	; VIA DRA
					; bit	function
					; ---	--------
					;  7	unused
					;  6	pin 07 P15 pendant, button row 6
					;  5	pin 06 P15 pendant, button row 5
					;  4	pin 05 P15 pendant, button row 4
					;  3	pin 04 P15 pendant, button row 3
					;  2	pin 03 P15 pendant, button row 2
					;  1	pin 02 P15 pentant, button row 1
					;  0	pin 01 P15 pendant, button row 0
LAB_7802	= LAB_7800+$02	; VIA DDRB
LAB_7803	= LAB_7800+$03	; VIA DDRA
LAB_7804	= LAB_7800+$04	; VIA T1Cl
LAB_7805	= LAB_7800+$05	; VIA T1Ch
LAB_7806	= LAB_7800+$06	; VIA T1Ll
LAB_7807	= LAB_7800+$07	; VIA T1Lh
LAB_7808	= LAB_7800+$08	; VIA T2Cl
LAB_780A	= LAB_7800+$0A	; VIA SR
LAB_780B	= LAB_7800+$0B	; VIA ACR
					; bit	function
					; ---	--------
					;  7	T1 PB7 enabled/disabled
					;  6	T1 free run/one shot
					;  5	T2 clock PB6/2
					; 432	function
					; ---	--------
					; 000	shift register disabled
					; 001	shift in, rate controlled by T2
					; 010	shift in, rate controlled by 2
					; 011	shift in, rate controlled by externak clock
					; 100	shift out, rate controlled by T2, free run mode
					; 101	shift out, rate controlled by T2
					; 110	shift out, rate controlled by 2
					; 111	shift out, rate controlled by externak clock
					;  1	PB latch enabled/disabled
					;  0	PA latch enabled/disabled
LAB_780C	= LAB_7800+$0C	; VIA PCR
					; bit	function
					; ---	--------
					; 765	CB2 control
					; ---	-----------
					; 000	Interrupt Input Mode
					; 001	Independent Interrupt Input Mode
					; 010	Input Mode
					; 011	Independent Input Mode
					; 100	Handshake Output Mode
					; 101	Pulse Output Mode
					; 110	Manual Output Mode, CB2 low
					; 111	Manual Output Mode, CB2 high
					;  4	CB1 edge positive/negative
					; 321	CA2 control
					; ---	-----------
					; 000	Interrupt Input Mode
					; 001	Independent Interrupt Input Mode
					; 010	Input Mode 
					; 011	Independent Input Mode
					; 100	Handshake Output Mode
					; 101	Pulse Output Mode
					; 110	Manual Output Mode, CA2 low
					; 111	Manual Output Mode, CA2 high
					;  0	CA1 edge positive/negative
LAB_780E	= LAB_7800+$0E	; VIA IER
					; bit	function
					; ---	--------
					;  7	enable/disable
					;  6	T1 interrupt
					;  5	T2 interrupt
					;  4	CB1 transition
					;  3	CB2 transition
					;  2	8 shifts done
					;  1	CA1 transition
					;  0	CA2 transition


;************************************************************************************
;
; system ROM, echoed from $8000

	.ORG	$F000			; ROM start

;LAB_F000:
	.byte	"(C) MICROBOT INC. 1982"
					; unused title string


;************************************************************************************
;
; clock divider for T2 a controlled shift register, only the low 8 bits of T2 are used
; T2 time is n+2 phase 2 clocks, one shift bit is 2 T2 periods
;
; for the first two n values a pattern that repeats every 8 register shifts is used,
; for the last six n values a pattern that repeats every 2 register shifts is used

LAB_F016:
	.byte	$3F			; 1843200/2/(63+2)/8/16 =  111
	.byte	$2E			; 1843200/2/(46+2)/8/16 =  150
	.byte	$5E			; 1843200/2/(94+2)/2/16 =  300
	.byte	$2E			; 1843200/2/(46+2)/2/16 =  600
	.byte	$16			; 1843200/2/(22+2)/2/16 = 1200
	.byte	$0A			; 1843200/2/(10+2)/2/16 = 2400
	.byte	$04			; 1843200/2/( 4+2)/2/16 = 4800
	.byte	$01			; 1843200/2/( 1+2)/2/16 = 9600

;##	.byte	$2E			; 1843200/2/(22+2)/8/ 1 =  2400
;##	.byte	$16			; 1843200/2/(22+2)/8/ 1 =  4800
;##	.byte	$2E			; 1843200/2/(46+2)/2/ 1 =  9600
;##	.byte	$1E			; 1843200/2/(30+2)/2/ 1 = 14400
;##	.byte	$16			; 1843200/2/(22+2)/2/ 1 = 19200
;##	.byte	$0A			; 1843200/2/(10+2)/2/ 1 = 38400
;##	.byte	$06			; 1843200/2/( 6+2)/2/ 1 = 57600
;##	.byte	$04			; 1843200/2/( 4+2)/2/ 1 = 76800


;************************************************************************************
;
; button value table

LAB_F01E:
	.byte	$00			; 0	TRAIN		B+
	.byte	$01			; 1	PAUSE		S+
	.byte	$02			; 2	GRIP		E+
	.byte	$03			; 3	OUT		P+
	.byte	$04			; 4	FREE		R+
	.byte	$FA			;	MOVE		G+
	.byte	$FB			;	MODE		STOP
	.byte	$05			; 5	STEP		B-
	.byte	$06			; 6	POINT		S-
	.byte	$07			; 7	JUMP		E-
	.byte	$08			; 8	CLEAR		P-
	.byte	$09			; 9	ZERO		R-
	.byte	$FD			;	SPEED		G-
	.byte	$FE			;	REC		RUN


;************************************************************************************
;
; SPEED values

LAB_F02C:
	.byte	$FF			;  0 slowest speed
	.byte	$90			;  1
	.byte	$60			;  2
	.byte	$48			;  3
	.byte	$32			;  4

; default SPEED value

LAB_F031:
	.byte	$22			;  5 default speed
	.byte	$17			;  6
	.byte	$13			;  7
	.byte	$11			;  8
	.byte	$10			;  9
	.byte	$0F			; 10
	.byte	$0E			; 11
	.byte	$0D			; 12
	.byte	$0C			; 13
	.byte	$0B			; 14
	.byte	$0A			; 15 fastest speed


;************************************************************************************
;
; RESET vector

LAB_F03C:
	LDA	#$40			; set IOII IIII
	STA	LAB_7802		; save VIA DDRB
	LDA	#$00			; set all inputs
	STA	LAB_7803		; save VIA DDRA
	STA	LAB_7800		; save VIA DRB

	LDA	#$50			; T1 continuous interrupts, T2 timed interrupt, shift out
					; free run at T2 rate, PB latch disable, PA latch disable
	STA	LAB_780B		; save VIA ACR
	LDA	#$EE			; CB2 high, CB1 negative edge, CA2 high, CA1 negative edge
	STA	LAB_780C		; save VIA PCR

; set up the ACIAs' clocks, uses T2 and the shift register

	LDA	LAB_7800		; read VIA DRB
	AND	#$07			; mask 0000 0xxx, the baud switches
	TAX				; copy it to the index
	LDA	LAB_F016,X		; get the T2 counter value from the table
	STA	LAB_7808		; save VIA T2Cl, only the low 8 bits of T2 are used
	LDA	#$AA			; set the shift register patttern for / 2 clock
	CPX	#$02			; compare the index with $02
	BCS	LAB_F067		; if >= $02 go use the / 2 pattern

	LDA	#$0F			; else set the shift register patttern for / 8 clock
LAB_F067:
	STA	LAB_780A		; save the VIA SR patttern

; setup the 6850 ACIAs

	LDA	#$03			; soft reset
	STA	LAB_4800		; reset ACIA A
	STA	LAB_5000		; reset ACIA B
	LDA	#$95			; setup for interrupts enabled, 8N1, clock/16
;##	LDA	#$94			; setup for interrupts enabled, 8N1, clock/1
	STA	LAB_4800		; set ACIA A
	STA	LAB_5000		; set ACIA B

; clear all the variables, LAB_E9 down to LAB_80

	LDA	#$00			; clear A
	LDX	#LAB_E9		; set the end address
LAB_F07E:
	STA	LAB_00,X		; clear a zero page byte
	DEX				; decrement the index
	BMI	LAB_F07E		; loop if more to do

; set the RAM top

	LDX	#<LAB_0400		; set the 1K RAM end low byte
	LDY	#>LAB_0400		; set the 1K RAM end high byte
	STY	LAB_07FF		; save the test byte
	CPY	LAB_07FF		; test the test byte
	BNE	LAB_F093		; if the test failed skip the 2K RAM set

	LDX	#<LAB_07FE		; set the 2K RAM end low byte
	LDY	#>LAB_07FE		; set the 2K RAM end high byte
LAB_F093:
	STX	LAB_A0		; save the RAM end low byte
	STY	LAB_A1		; save the RAM end high byte

; the stack pointer isn't set yet and the following subroutine writes over parts of page
; one, the stack page

	JSR	LAB_F15D		; clear all the program steps

; initialise the stepper drive patterns

	LDA	#$01			; set the fill byte, this is the first byte of the half
					; step sequence
	LDX	#$05			; set the stepper count/index
LAB_F09E:
	STA	LAB_E3,X		; set a stepper drive byte
	DEX				; decrement the stepper count/index
	BPL	LAB_F09E		; loop if more to do

; set the recognition character

	LDA	#'@'			; set the recognition character
	STA	LAB_88		; save the recognition character

; clear all the bits of the 74LS259, 8 bit addressable latch

	LDX	#$07			; set the byte count/index
LAB_F0A9:
	STA	LAB_1C00,X		; clear a latch bit
	DEX				; decrement byte the count/index
	BPL	LAB_F0A9		; loop if more to do

	LDA	#$FF			; set T1 low byte
	STA	LAB_7804		; save VIA T1Cl

	LDA	LAB_F031		; get the default SPEED value
	STA	LAB_AC		; save the SPEED value

	LDA	#$C0			; enable T1 interrupt
	STA	LAB_780E		; save VIA IER

; restart the main loop

LAB_F0BE:
	LDX	#<LAB_0119		; set the stack address low byte
	TXS				; set the stack pointer
	CLI				; enable the interrupts

; set TRAIN mode

LAB_F0C2:
	LDA	#$F1			; set TRAIN mode, set the LED to off
LAB_F0C4:
	STA	LAB_1C00		; save the MODE LED
	STA	LAB_1C07		; save the RUN LED
LAB_F0CA:
	LDX	LAB_83		; get the last held button
	BNE	LAB_F0CA		; if a button is held go wait some more

	STA	LAB_80		; save the mode byte
	CMP	#$F1			; compare the mode with TRAIN mode
	BNE	LAB_F0D9		; if not TRAIN mode skip setting the LED on

	LDA	#$00			; set the LED to on
	STA	LAB_1C06		; save the TRAIN LED
LAB_F0D9:
	LDA	LAB_AC		; get the SPEED value
	JSR	LAB_F848		; set T1Ch and the button scan wait count

; main loop

LAB_F0DE:
	LDA	LAB_86		; get the line received flag
	BEQ	LAB_F0E5		; if no line has been received go try the pendant

	JMP	LAB_F2C4		; else go interpret commands from the serial port

; interpret commands from the pendant

LAB_F0E5:
	JSR	LAB_F9A8		; get a button
	BEQ	LAB_F0DE		; if no button return to the main loop

; there was a button

	LDX	#$01			; set the LED to off
	STX	LAB_1C06		; save the TRAIN LED

	LDX	LAB_9A		; get the @seT mode flag
	BEQ	LAB_F0FA		; if not @seT mode continue

; else clear @seT mode and return to the main loop

	LDX	#$00			; clear X
	STX	LAB_9A		; clear the @seT mode flag
	JMP	LAB_F475		; go return ok message and enter TRAIN mode

; is not @seT mode

LAB_F0FA:
	EOR	#$07			; compare it with the MODE button
	BNE	LAB_F102		; if it wasn't MODE continue

; it was MODE so clear the last held button press and skip the parameter copy

	STA	LAB_83		; clear the last held button
	BEQ	LAB_F120		; go wait for a MODE to be set, branch always

; copy all the parameters to the program step

LAB_F102:
	LDY	#$0D			; set the byte count/index
LAB_F104:
	LDA	LAB_AC-1,Y		; get a parameter byte
	STA	(LAB_8D),Y		; save it to the program memory
	DEY				; decrement the byte count/index
	BNE	LAB_F104		; loop if more to do

	LDA	#$01			; set move with acceleration control command
	STA	(LAB_8D),Y		; save the command byte
	JSR	LAB_F1CF		; increment the the program step pointer
	STA	LAB_9F		; set the no increment flag
	BNE	LAB_F0C2		; go set TRAIN mode and retun to the main loop, branch
					; always

; turn the MODE LED off and wait for no button press

LAB_F117:
	LDA	#$01			; set the LED to off
	STA	LAB_1C00		; save the MODE LED

; wait for no button press

LAB_F11C:
	LDA	LAB_83		; get the last held button
	BNE	LAB_F11C		; if a button is held go wait some more

; wait for a MODE to be set

LAB_F120:
	LDX	#$01			; set the LED to off
	STX	LAB_1C07		; save the RUN LED
	STX	LAB_1C06		; save the TRAIN LED
	STX	LAB_7800		; save VIA DRB, ENTER LED off (b6)

	DEX				; clear X, set the LED to on
	STX	LAB_1C00		; save the MODE LED
	STX	LAB_80		; clear the mode byte
LAB_F131:
	JSR	LAB_F9A8		; get a button
	BNE	LAB_F13D		; if a button was pressed go handle it

	LDA	LAB_86		; get the line received flag
	BEQ	LAB_F131		; if no line has been received go wait some more

	JMP	LAB_F2C4		; go interpret commands from the serial port

; a button was pressed

LAB_F13D:
	CMP	#$07			; compare the button with MODE
	BNE	LAB_F175		; if not MODE continue

; the button was MODE

LAB_F141:
	LDA	LAB_83		; get the last held button
	BEQ	LAB_F131		; if no button held loop for another button

	LDA	LAB_A3		; get the pendant column 5 state
	CMP	#$EF			; compare it with ZERO pressed
	BEQ	LAB_F155		; if it is ZERO go clear all positions and the program

	CMP	#$F7			; compare it with CLEAR pressed
	BNE	LAB_F141		; if not clear go wait some more

	JSR	LAB_F15D		; clear all the program steps
LAB_F152:
	JMP	LAB_F117		; turn the MODE LED off and wait for no button press

; MODE held and ZERO pressed

LAB_F155:
	JSR	LAB_F995		; reset the program step pointer
	JSR	LAB_F29F		; clear all position registers
	BMI	LAB_F152		; turn the MODE LED off and wait for no button press,
					; branch always


;************************************************************************************
;
; clear all the program steps

LAB_F15D:
	JSR	LAB_F995		; reset the program step pointer
	LDY	#$00			; clear Y
LAB_F162:
	TYA				; clear A
	STA	(LAB_8D),Y		; clear the program step command byte
	JSR	LAB_F1CF		; increment the the program step pointer
	LDA	LAB_8D		; get the the program step pointer low byte
	CMP	#<LAB_011A		; compare it with the program memory start low byte
	BNE	LAB_F162		; if not there yet go clear another step

	LDA	LAB_8E		; get the the program step pointer high byte
	CMP	#>LAB_011A		; compare it with the program memory start high byte
	BNE	LAB_F162		; if not there yet go clear another step

	RTS


;************************************************************************************
;
; a button was pressed, continued

LAB_F175:
	CMP	#$0D			; compare the button with SPEED
	BNE	LAB_F189		; if not SPEED continue

; the button was SPEED

	JSR	LAB_F914		; get a numeric value from the buttons
	AND	#$0F			; mask 0000 xxxx, 16 values
	TAX				; copy it to the index
	LDA	LAB_F02C,X		; get the indexed SPEED value
	STA	LAB_AC		; save the SPEED value
	JSR	LAB_F848		; set T1Ch and the button scan wait count
	BNE	LAB_F120		; go wait for a MODE to be set, branch always

; a button was pressed, continued

LAB_F189:
	CMP	#$04			; compare the button with OUT
	BNE	LAB_F1EA		; if not OUT continue

; the button was OUT

	LDY	#$00			; clear the index
	LDA	#$05			; set OUT command
	STA	(LAB_8D),Y		; save the command byte

	JSR	LAB_F914		; get a numeric value from the buttons
	LDY	#$01			; set the index
	STA	(LAB_8D),Y		; save the item number
	JSR	LAB_F914		; get a numeric value from the buttons
	LDY	#$02			; set the index
	STA	(LAB_8D),Y		; save 1 for on and 0 for off

	DEY				; set the index to the item number
	LDA	(LAB_8D),Y		; get the item number
	TAX				; copy it
	LDA	LAB_84		; get the numeric value low byte
	JSR	LAB_F1B4		; set item X to state A

; increment the the program step pointer and wait for a MODE to be set

LAB_F1AA:
	JSR	LAB_F1CF		; increment the the program step pointer

; set the no increment flag and wait for a MODE to be set

LAB_F1AD:
	LDY	#$FF			; set X
	STY	LAB_9F		; set the no increment flag
	JMP	LAB_F120		; go wait for a MODE to be set


;************************************************************************************
;
; set item X to state A

LAB_F1B4:
	CPX	#$08			; compare the item number with the ENTER LED
	BNE	LAB_F1C1		; if not the ENTER LED continue

; else it was item 8, the ENTER LED

	ROR				; shift the state ..
	ROR				; .. bit through the ..
	ROR				; .. carry to b6
	AND	#$40			; mask 0x00 0000, the ENTER LED
	STA	LAB_7800		; save VIA DRB
	RTS

; was not item 8

LAB_F1C1:
	CPX	#$00			; compare it with item 0, the MODE LED
	BEQ	LAB_F1C9		; if it is item 0 go toggle the state bit

	CPX	#$06			; compare it with item 6, the TRAIN LED
	BCC	LAB_F1CB		; if < the TRAIN LED just use the state bit

; it was item 0, 6 or 7, the ENTER, TRAIN or RUN LED

LAB_F1C9:
	EOR	#$FF			; toggle the state bit

; items 1 to 5

LAB_F1CB:
	STA	LAB_1C00,X		; set item X to state A
	RTS


;************************************************************************************
;
; increment the the program step pointer

LAB_F1CF:
	CLC				; clear carry for add
	LDA	LAB_8D		; get the the program step pointer low byte
	ADC	#$0E			; add the program step length
	STA	LAB_8D		; save the the program step pointer low byte
	BCC	LAB_F1DC		; if no carry skip the high byte increment

	INC	LAB_8E		; else increment the the program step pointer high byte
	LDA	LAB_8D		; get the the program step pointer low byte, not needed
LAB_F1DC:
	CMP	LAB_A0		; compare it with the RAM end low byte
	BNE	LAB_F1E9		; if not there yet just exit

	LDA	LAB_8E		; get the the program step pointer high byte
	CMP	LAB_A1		; compare it with the RAM end high byte
	BNE	LAB_F1E9		; if not there yet just exit

	JSR	LAB_F995		; else reset the program step pointer
LAB_F1E9:
	RTS


;************************************************************************************
;
; a button was pressed, continued

LAB_F1EA:
	CMP	#$02			; compare the button with PAUSE
	BNE	LAB_F1FE		; if not PAUSE continue

; the button was PAUSE

	LDY	#$00			; clear the index
	LDA	#$04			; set PAUSE command
	STA	(LAB_8D),Y		; save the command byte
	JSR	LAB_F914		; get a numeric value from the buttons
	LDY	#$01			; set the index
	STA	(LAB_8D),Y		; set the pause value
	JMP	LAB_F1AA		; go increment the the program step pointer and wait for
					; a MODE to be set

; a button was pressed, continued

LAB_F1FE:
	CMP	#$0A			; compare the button with JUMP
	BNE	LAB_F21E		; if not JUMP continue

; the button was JUMP

	LDY	#$00			; clear the index
	LDA	#$03			; set JUMP command
	STA	(LAB_8D),Y		; save the command byte
	JSR	LAB_F914		; get a numeric value from the buttons
	LDY	#$01			; set the index
	STA	(LAB_8D),Y		; save the jump condition
	JSR	LAB_F914		; get a numeric value from the buttons
	LDY	#$02			; set the index
	STA	(LAB_8D),Y		; save the jump value low byte
	INY				; increment the index
	LDA	LAB_85		; get the numeric value high byte
	STA	(LAB_8D),Y		; save the jump value high byte
	JMP	LAB_F1AA		; go increment the the program step pointer and wait for
					; a MODE to be set

; a button was pressed, continued

LAB_F21E:
	CMP	#$06			; compare the button with MOVE
	BNE	LAB_F227		; if not MOVE continue

; the button was MOVE

	LDA	#$FF			; set MOVE mode, set the LED to off
	JMP	LAB_F0C4		; save the mode and return to the main loop

; a button was pressed, continued

LAB_F227:
	CMP	#$01			; compare the button with TRAIN
	BNE	LAB_F22E		; if not TRAIN continue

; the button was TRAIN

	JMP	LAB_F0C2		; go set TRAIN mode and return to the main loop

; a button was pressed, continued

LAB_F22E:
	CMP	#$0E			; compare the button with RUN
	BNE	LAB_F250		; if not RUN continue

; the button was RUN

LAB_F232:
	LDY	#$01			; set the LED to off
	STY	LAB_1C00		; save the MODE LED
	DEY				; set the LED to on
	STY	LAB_1C07		; save the RUN LED
LAB_F23B:
	JSR	LAB_F737		; interpret a program step
	JSR	LAB_F9A8		; get a button
	CMP	#$07			; compare the button with MODE
	BEQ	LAB_F29C		; if it is MODE exit running program steps and go wait for
					; a MODE to be set

	CMP	#$0E			; compare the button with RUN
	BEQ	LAB_F29C		; if it is RUN exit running program steps and go wait for
					; a MODE to be set

	LDA	LAB_86		; get the line received flag
	BEQ	LAB_F23B		; if no line has been received go run for a bit longer

	JMP	LAB_F2C4		; go interpret commands from the serial port

; a button was pressed, continued

LAB_F250:
	CMP	#$08			; compare the button with STEP
	BNE	LAB_F263		; if not STEP continue

; the button was STEP

	LDY	#$01			; set the LED to off
	STY	LAB_1C00		; save the MODE LED
	DEY				; set the LED to on
	STY	LAB_1C07		; save the RUN LED
	JSR	LAB_F737		; interpret a program step
	JMP	LAB_F11C		; wait for no button press

; a button was pressed, continued

LAB_F263:
	CMP	#$05			; compare the button with FREE
	BNE	LAB_F27C		; if not FREE continue

; the button was FREE

	JSR	LAB_F26D		; clear all the 74LS174 hex D-type flip flops
	JMP	LAB_F117		; turn the MODE LED off and wait for no button press


;************************************************************************************
;
; clear all the 74LS174 hex D-type flip flops

LAB_F26D:
	LDA	#$00			; clear A
	STA	LAB_0800		; clear 74LS174 hex D-type flip flop 0
	STA	LAB_0C00		; clear 74LS174 hex D-type flip flop 1
	STA	LAB_1000		; clear 74LS174 hex D-type flip flop 2
	STA	LAB_1400		; clear 74LS174 hex D-type flip flop 3
	RTS


;************************************************************************************
;
; a button was pressed, continued

LAB_F27C:
	CMP	#$03			; compare the button with GRIP
	BNE	LAB_F2B5		; if not GRIP continue

; the button was GRIP

	LDY	#$00			; clear the index, set the LED to on
	STY	LAB_1C07		; save the RUN LED
	LDA	#$02			; set GRIP command
	STA	(LAB_8D),Y		; save the command byte
	INY				; set the LED to off
	STY	LAB_1C00		; save the MODE LED
	LDA	LAB_AC		; get the SPEED value
	STA	(LAB_8D),Y		; save the SPEED value
	JSR	LAB_F96D		; close the gripper and $20 steps more
	JSR	LAB_F1CF		; increment the the program step pointer
	STA	LAB_9F		; set the no increment flag
	JMP	LAB_F11C		; wait for no button press


;************************************************************************************
;
; go wait for a MODE to be set

LAB_F29C:
	JMP	LAB_F120		; go wait for a MODE to be set


;************************************************************************************
;
; clear all position registers
;
; adds the current position low byte to the phase offset so that any phase can be the
; zero position

LAB_F29F:
	LDX	#$05			; set the stepper count/index
LAB_F2A1:
	CLC				; clear carry for add
	LDA	LAB_DD,X		; get the stepper x phase offset
	ADC	LAB_AD,X		; add the stepper x count low byte
	STA	LAB_DD,X		; save the stepper x phase offset
	DEX				; decrement the stepper count/index
	BPL	LAB_F2A1		; loop if more to do

	LDA	#$00			; clear A
	LDX	#$0B			; set the count/index
LAB_F2AF:
	STA	LAB_AD,X		; clear a stepper x count byte
	DEX				; decrement the count/index
	BPL	LAB_F2AF		; loop if more to do

	RTS


;************************************************************************************
;
; a button was pressed, continued

LAB_F2B5:
	CMP	#$09			; compare the button with POINT
	BNE	LAB_F29C		; if not POINT go wait for a MODE to be set

; the button was POINT

	JSR	LAB_F914		; get a numeric value from the buttons
	LDX	LAB_85		; get the numeric value high byte
	JSR	LAB_F797		; set the program step pointer to step XA
	JMP	LAB_F1AD		; go set the no increment flag and wait for a MODE to be
					; set


;************************************************************************************
;
; interpret commands from the serial port

LAB_F2C4:
	LDA	#$00			; clear A
	STA	LAB_86		; clear the line received flag
LAB_F2C8:
	JSR	LAB_F9B3		; get a byte from the serial buffer
	CMP	LAB_88		; compare it with the recognition character
	BNE	LAB_F2C8		; if it isn't the recognition character go get another

	LDA	LAB_AA		; get the command sequence error flag
	BEQ	LAB_F2D9		; if there was no error go process the command string

; there was an error in the command string

	LDA	#$00			; clear A
	STA	LAB_AA		; clear the command sequence error flag
	BEQ	LAB_F300		; go decrement the Rx buffer read index and return the bad
					; syntax message, branch always

; wait for the recognition character then skip the next two characters and test the
; third one. the third character is unique in each command

LAB_F2D9:
	LDY	#$02			; set the character count
LAB_F2DB:
	JSR	LAB_F9B3		; get a byte from the serial buffer
	CMP	LAB_88		; compare it with the recognition character
	BEQ	LAB_F2D9		; if it is the recognition character go reset the count

	AND	#$7F			; mask 0xxx xxxx, 7 bit ASCII
	CMP	#$0D			; compare the byte with [CR]
	BEQ	LAB_F300		; if it is [CR] go decrement the Rx buffer read index and
					; return the bad syntax message

	DEY				; decrement the character count
	BPL	LAB_F2DB		; loop if more characters to go

	CMP	#'M'			; compare the byte with "M"
	BNE	LAB_F317		; if not "M" continue

; the command was @arM <chr>
;
; this command changes the recognition character, the new character is the character
; following the [SPACE] chracter following the @ARM command as long as that character
; isn't [CR]

LAB_F2EF:
	JSR	LAB_F9B3		; get a byte from the serial buffer
	CMP	LAB_88		; compare it with the recognition character
	BEQ	LAB_F2D9		; if it is the recognition character go reset the count

	AND	#$7F			; mask 0xxx xxxx, 7 bit ASCII
	CMP	#' '			; compare it with [SPACE]
	BEQ	LAB_F30B		; if it was [SPACE] go continue the command

	CMP	#$0D			; compare the byte with [CR]
	BNE	LAB_F2EF		; if it wasn't [CR] go get more characters

; decrement the Rx buffer read index and return the bad syntax message

LAB_F300:
	DEC	LAB_8C		; decrement the Rx buffer read index
	LDA	LAB_8C		; get the Rx buffer read index
	AND	#$7F			; mask 0xxx xxxx, the buffer extent
	STA	LAB_8C		; save the Rx buffer read index
	JMP	LAB_F46E		; go return the bad syntax message and enter TRAIN mode

; continue the @arM command

LAB_F30B:
	JSR	LAB_F9B3		; get a byte from the serial buffer
	CMP	#$0D			; compare the byte with [CR]
	BEQ	LAB_F300		; if it is [CR] go decrement the Rx buffer read index and
					; return the bad syntax message

	STA	LAB_88		; save the recognition character
LAB_F314:
	JMP	LAB_F475		; go return ok message and enter TRAIN mode

; interpret commands continued

LAB_F317:
	INY				; set the mode to moving, Y = $01
	STY	LAB_80		; save the mode byte

	CMP	#'O'			; compare the byte with "O"
	BNE	LAB_F329		; if not "O" continue

; the command was @clOse <spd>
;
; this command closes the gripper just to the point where the grip switch closes

	JSR	LAB_F8ED		; input a signed decimal value, set T1Ch and the button
					; scan wait count
	LDA	#$01			; set the number of steps to go past the switch point
	JSR	LAB_F96F		; close the gripper and A steps more
	JMP	LAB_F363		; go return 'ok' or 'stopped'

; interpret commands continued

LAB_F329:
	CMP	#'E'			; compare the byte with "E"
	BNE	LAB_F36C		; if not "E" continue

; the command was @stEp <spd>,<m0>,<m1>,<m2>,<m3>,<m4>,<m5>,<out>
;
; this command moves all six motors at the speed specified by the amount set. the
; outputs are set by the <out> value

	JSR	LAB_F8ED		; input a signed decimal value, set T1Ch and the button
					; scan wait count
	LDX	#$00			; clear the stepper index
LAB_F332:
	STX	LAB_9B		; save the stepper index
	JSR	LAB_F859		; input a signed decimal value
	LDX	LAB_9B		; get the stepper index
	STY	LAB_B9,X		; save the stepper x delta low byte
	STA	LAB_BF,X		; save the stepper x delta high byte
	INX				; increment the stepper index
	CPX	#$06			; compare it with the max + 1
	BCC	LAB_F332		; loop if more to do

	LDA	LAB_9C		; get the digit received flag
	BEQ	LAB_F35E		; if no digits skip the last get

; so get the value for the <out> part of the command

	JSR	LAB_F859		; input a signed decimal value
	ROR				; shift the state ..
	ROR				; .. bit through the ..
	ROR				; .. carry to b6
	AND	#$40			; mask 0x00 0000, the ENTER LED
	STA	LAB_7800		; save VIA DRB

	LDX	#$07			; set the bit count/index
	TYA				; copy the value low byte
	EOR	#$C1			; toggle xx.. ...x, the LED bits
	ROL				; shift bit 7 to carry
LAB_F357:
	ROL				; shift next bit to bit 0
	STA	LAB_1C00,X		; save the bit to the addressable latch
	DEX				; decrement the bit count/index
	BPL	LAB_F357		; loop if more to do

LAB_F35E:
	STX	LAB_A4		; set the acceleration control flag, X <> $00
	JSR	LAB_F502		; setup the values for a move
LAB_F363:
	CMP	#$07			; compare the byte returned with STOP
	BNE	LAB_F314		; if not STOP go return ok message and enter TRAIN mode

	LDX	#'2'			; else set "2", stopped
	JMP	LAB_F470		; go return the CHR$(X) message and enter TRAIN mode

; interpret commands continued

LAB_F36C:
	CMP	#'A'			; compare the byte with "A"
	BNE	LAB_F3A5		; if not "A" continue

; the command was @reAd
;
; this command returns the internal position count of all six motors followed by a
; number epresenting the eight binary inputs plus the last button pressed * 256. if
; no button has been pressed since last time this part will be zero otherwise it will
; be 256 * n where n is one of the following
;
;  1		B+		TRAIN		0
;  2		S+		PAUSE		1
;  3		E+		GRIP		2
;  4		P+		OUT		3
;  5		R+		FREE		4
;  6		G+		MOVE
;  7		STOP		MODE
;  8		B-		STEP		5
;  9		S-		POINT		6
; 10		E-		JUMP		7
; 11		P-		CLEAR		8
; 12		R-		ZERO		9
; 13		G-		SPEED	
; 14		RUN		REC

	JSR	LAB_F47F		; respond ok

	LDX	#$00			; clear the stepper index
LAB_F375:
	STX	LAB_9B		; save the stepper index
	LDY	LAB_AD,X		; get the stepper x count low byte
	LDA	LAB_B3,X		; get the stepper x count high byte
	JSR	LAB_F9BF		; output AY as a signed, leading zero supressed, integer
	LDX	#','			; set ","
	JSR	LAB_F8FC		; send ACIA A byte with delay
	LDX	LAB_9B		; restore the stepper index
	INX				; increment it to the next stepper
	CPX	#$06			; compare with max + 1
	BCC	LAB_F375		; loop if more to do

	LDX	#$07			; set the bit count/index
LAB_F38C:
	LDA	LAB_1800,X		; read the 74LS251 data selector bit X
	ASL				; shift it out to the carry ..
	ROL	LAB_97		; .. and into the data selector byte
	DEX				; decrement the bit count/index
	BPL	LAB_F38C		; loop if more to do

	LDA	LAB_97		; get the data selector byte
	EOR	#$FF			; toggle all the bits
	TAY				; copy it to the word low byte
	LDA	LAB_82		; get the @READ last button pressed
	INX				; clear X
	STX	LAB_82		; clear the @READ last button pressed
	JSR	LAB_F9BF		; output AY as a signed, leading zero supressed, integer
	JMP	LAB_F469		; send [CR] and enter TRAIN mode

; interpret commands continued

LAB_F3A5:
	CMP	#'T'			; compare the byte with "T"
	BNE	LAB_F3B9		; if not "T" continue

; the command was @seT <spd>
;
; this command activates the pendant in train mode meaning an arm position can be set
; with the buttons. the arm wil remain in TRAIN mode until MODE or REC is pressed

	JSR	LAB_F8ED		; input a signed decimal value, set T1Ch and the button
					; scan wait count
	STA	LAB_9A		; set the @seT mode flag, A <> $00 here
	LDA	LAB_A6		; get the T1Ch value
	STA	LAB_A8		; save the timer 1 minimum value high byte
	LDA	#$F1			; set TRAIN mode
	STA	LAB_80		; save the mode byte
	JMP	LAB_F0E5		; return to the main loop

; interpret commands continued

LAB_F3B9:
	CMP	#'S'			; compare the byte with "S"
	BNE	LAB_F3CD		; if not "S" continue

; the command was @reSet
;
; this command clears all the internal position counters and turns off the motors
; allowing the arm to be manually positioned

	LDA	#$00			; clear A
	STA	LAB_80		; clear the mode byte
	JSR	LAB_F26D		; clear all the 74LS174 hex D-type flip flops
	JSR	LAB_F29F		; clear all position registers
	JSR	LAB_F47F		; respond ok
	JMP	LAB_F120		; go wait for a MODE to be set

; interpret commands continued

LAB_F3CD:
	CMP	#'L'			; compare the byte with "L"
	BNE	LAB_F3D9		; if not "L" continue

; the command was @deLay <n>
;
; this command inserts a delay of approximately n/2 milliseconds before each character
; is sent over the serial port from the arm

	JSR	LAB_F859		; input a signed decimal value
	STY	LAB_A2		; save the input value low byte as the Tx character delay
	JMP	LAB_F475		; go return ok message and enter TRAIN mode

; interpret commands continued

LAB_F3D9:
	CMP	#'U'			; compare the byte with "U"
	BNE	LAB_F42D		; if not "U" continue

; the command was @qdUmp
;
; this command dumps all 53 (126 on a 2K system) program steps as a number followed
; by seven signed decimal word values read from the program step. these values are
; all separated by commas

	JSR	LAB_F47F		; respond ok
	JSR	LAB_F995		; reset the program step pointer

	LDX	#$00			; clear X
	STX	LAB_94		; clear the program step number
LAB_F3E7:
	LDA	#$07			; set the word count
	STA	LAB_9B		; save the word count

	LDY	LAB_94		; get the program step number
	LDA	#$00			; clear the high byte
	JSR	LAB_F9BF		; output AY as a signed, leading zero supressed, integer
	LDX	#','			; set ","
	JSR	LAB_F8FC		; send ACIA A byte with delay

LAB_F3F7:
	LDX	#$00			; clear the index
	LDA	(LAB_8D,X)		; get the value low byte
	TAY				; copy it
	INC	LAB_8D		; increment the the program step pointer low byte
	LDA	(LAB_8D,X)		; get the value high byte
	JSR	LAB_F9BF		; output AY as a signed, leading zero supressed, integer
	INC	LAB_8D		; increment the the program step pointer low byte
	BNE	LAB_F409		; if no rollover skip the high byte increment

	INC	LAB_8E		; else increment the the program step pointer high byte
LAB_F409:
	DEC	LAB_9B		; decrement the word count
	BEQ	LAB_F414		; if all done go output a [CR]

	LDX	#','			; set ","
	JSR	LAB_F8FC		; send ACIA A byte with delay
	BNE	LAB_F3F7		; go do another value, branch always

; reached the end of a program step

LAB_F414:
	LDX	#$0D			; set [CR]
	JSR	LAB_F8FC		; send ACIA A byte with delay
	INC	LAB_94		; increment the program step number
	LDA	LAB_8D		; get the the program step pointer low byte
	CMP	LAB_A0		; compare it with the RAM end low byte
	BNE	LAB_F3E7		; if not there yet go do another program step

	LDA	LAB_8E		; get the the program step pointer high byte
	CMP	LAB_A1		; compare it with the RAM end high byte
	BNE	LAB_F3E7		; if not there yet go do another program step

; dumped all the program steps

	JSR	LAB_F995		; reset the program step pointer
	JMP	LAB_F478		; enter TRAIN mode

; interpret commands continued

LAB_F42D:
	CMP	#'R'			; compare the byte with "R"
	BNE	LAB_F457		; if not "R" continue

; the command was @qwRite <n>,<v0>,<v1>,<v2>,<v3>,<v4>,<v5>,<v6>
;
; this command takes a number, n from 0 to 52 (0 to 125 on a 2K system) and writes the
; following seven values to the memory for that number program step. these values are
; the same format as those sent by the QDUMP command. v1, v2 and v3 are stepper 1, 0,
; 3, 2, 5, and 4's low bytes and v4, v5 and v5 are stepper 1, 0, 3, 2, 5, and 4's high
; bytes. v0 is the speed and command bytes

	JSR	LAB_F859		; input a signed decimal value
	TAX				; copy the input value high byte
	TYA				; copy the input value low byte
	JSR	LAB_F797		; set the program step pointer to step XA

	LDY	#$00			; clear the index
LAB_F43B:
	STY	LAB_9B		; save the index
	JSR	LAB_F859		; input a signed decimal value
	PHA				; save the input value high byte
	TYA				; copy the input value low byte
	LDY	LAB_9B		; read the index
	STA	(LAB_8D),Y		; save the input value low byte
	INC	LAB_9B		; increment the index
	LDY	LAB_9B		; read the index
	PLA				; restore the input value high byte
	STA	(LAB_8D),Y		; save the input value high byte
	LDY	LAB_9B		; read the index
	INY				; increment the index
	CPY	#$0E			; compare the index with the max + 1
	BCC	LAB_F43B		; loop if more to do

	JMP	LAB_F475		; go return ok message and enter TRAIN mode

; interpret commands continued

LAB_F457:
	CMP	#'N'			; compare the byte with "N"
	BNE	LAB_F46E		; if not "N" go return the bad syntax message and enter
					; TRAIN mode

; the command was @ruN <n>
;
; this command will run any program stored in the system starting from the step given.
; this can be steps 0 to 52 (0 to 125 on a 2K system) in RAM or 126 and above for the
; permanent demonstration program steps

	JSR	LAB_F859		; input a signed decimal value
	TAX				; copy the input value high byte
	TYA				; copy the input value low byte
	JSR	LAB_F797		; set the program step pointer to step XA
	JSR	LAB_F47F		; respond ok
	JMP	LAB_F232		; go do RUN

; send [CR] and enter TRAIN mode

LAB_F469:
	JSR	LAB_F494		; send [CR] via ACIA A byte with delay
	BNE	LAB_F478		; go enter TRAIN mode, branch always

; return the bad syntax message and enter TRAIN mode

LAB_F46E:
	LDX	#'0'			; set "0", bad syntax

; return the CHR$(X) message and enter TRAIN mode

LAB_F470:
	JSR	LAB_F481		; respond CHR$(X)
	BNE	LAB_F478		; skip the respond ok, branch always

; return ok message and enter TRAIN mode

LAB_F475:
	JSR	LAB_F47F		; respond ok

; enter TRAIN mode

LAB_F478:
	LDA	#$F1			; set TRAIN mode
	STA	LAB_80		; save the mode byte
	JMP	LAB_F0D9		; go set the speed and return to the main loop


;************************************************************************************
;
; respond ok

LAB_F47F:
	LDX	#'1'			; set "1"


;************************************************************************************
;
; respond CHR$(X)

LAB_F481:
	JSR	LAB_F8FC		; send ACIA A byte with delay
	LDX	LAB_8C		; get the Rx buffer read index
	LDA	LAB_00,X		; get an Rx buffer byte
LAB_F488:
	AND	#$7F			; mask 0xxx xxxx, 7 bit ASCII
	CMP	#$0D			; compare the byte with [CR]
	BEQ	LAB_F494		; if [CR] go echo the character and return

	JSR	LAB_F9B3		; get a byte from the serial buffer
	JMP	LAB_F488		; loop for the next byte


;************************************************************************************
;
; send [CR] via ACIA A byte with delay

LAB_F494:
	LDX	#$0D			; set [CR]
	JMP	LAB_F8FC		; send ACIA A byte with delay


;************************************************************************************
;
; IRQ vector, triggered by the ACIAs

LAB_F499:
	PHA				; save A
	TXA				; copy X
	PHA				; save X

	LDA	LAB_4800		; read ACIA A status
	BMI	LAB_F4B8		; if ACIA A caused the iterrupt go handle it

	LDA	LAB_5000		; read ACIA B status
	AND	#$7C			; mask 0xxx xx00, the error bits
	BNE	LAB_F4B2		; if there is an error go clear it and exit

; theer was no error so get the character and pass it on via ACIA A

	LDX	LAB_5001		; read the ACIA B Rx data byte 
	CLI				; enable interrupts
	JSR	LAB_F909		; send ACIA A byte
	JMP	LAB_F4CF		; restore X and A and return from interrupt

; clear the ACIA B error and exit

LAB_F4B2:
	LDA	LAB_5001		; read the ACIA B Rx data byte
	JMP	LAB_F4CF		; restore X and A and return from interrupt

; handle an ACIA A interrupt

LAB_F4B8:
	AND	#$7C			; mask 0xxx xx00, the error bits
	BNE	LAB_F4D2		; if there is an error go clear it and exit

	LDA	LAB_4801		; read the ACIA A Rx data byte
	CMP	LAB_88		; compare it with the recognition character
	BNE	LAB_F4DD		; if it isn't the recognition character go handle it

; the character was the recogition character so flag it and add it to the Rx buffer

	DEC	LAB_87		; set the recognition character received flag
LAB_F4C5:
	LDX	LAB_8B		; get the Rx buffer write index
	STA	LAB_00,X		; save the received byte
	INX				; increment the Rx buffer write index
	TXA				; copy it
	AND	#$7F			; mask 0xxx xxxx, 7 bit ASCII
	STA	LAB_8B		; save the Rx buffer write index
LAB_F4CF:
	JMP	LAB_F719		; restore X and A and return from interrupt

LAB_F4D2:
	LDX	LAB_4801		; read the ACIA A Rx data byte
	BIT	LAB_87		; test the recognition character received flag
	BPL	LAB_F4CF		; if no recognition character received go restore X and A
					; and return from interrupt

	STA	LAB_AA		; set the command sequence error flag
	BMI	LAB_F4CF		; restore X and A and return from interrupt, brach always

; the character wasn't the recogition character so see if it needs copying to the Rx
; buffer or passing on

LAB_F4DD:
	BIT	LAB_87		; test the recognition character received flag
	BMI	LAB_F4F0		; if recognition character received go test for [EOL]

; the recogition character hasn't been received so echo the character to any other
; devices through ACIA B

	CLI				; enable interrupts
	TAX				; copy the Tx data byte
	LDA	#$02			; mask 0000 00x0, ACIA B Tx status bit
LAB_F4E5:
	BIT	LAB_5000		; test the ACIA B Tx status bit
	BEQ	LAB_F4E5		; loop while the Tx buffer is full

	STX	LAB_5001		; save the ACIA B Tx data byte
	JMP	LAB_F4CF		; restore X and A and return from interrupt

; test for [EOL] and flag it if it is detected

LAB_F4F0:
	CMP	#$0D			; compare the byte with [CR]
	BEQ	LAB_F4FA		; if it is [CR] go flag a line has been received

	CMP	#$8D			; compare the byte with [CR] + $80
	BNE	LAB_F4C5		; if it is not [CR] go save it to the buffer

	AND	#$7F			; mask 0xxx xxxx, 7 bit ASCII
LAB_F4FA:
	DEC	LAB_86		; set the line received flag
	LDX	#$00			; clear X
	STX	LAB_87		; clear the recognition character received flag
	BEQ	LAB_F4C5		; go add the character to the buffer and exit, branch
					; always


;************************************************************************************
;
; setup the values for a move

; first find the largest delta for all the steppers

LAB_F502:
	LDX	#$05			; set the stepper count/index
	LDY	#$00			; clear Y
	STY	LAB_8F		; clear the maximum delta low byte
	STY	LAB_90		; clear the maximum delta high byte
LAB_F50A:
	STY	LAB_C5,X		; clear the stepper x direction low byte
	STY	LAB_CB,X		; clear the stepper x direction high byte

	LDA	LAB_BF,X		; get the stepper x delta high byte
	BMI	LAB_F515		; go handle a negative movement delta

	JMP	LAB_F597		; go handle a positive movement delta

; handle a negative delta

LAB_F515:
	DEC	LAB_C5,X		; decrement the stepper x direction low byte
	DEC	LAB_CB,X		; decrement the stepper x direction high byte
					; this makes the stepper direction word - 1

; now negate the delta

	SEC				; set carry for subtract
	TYA				; clear A
	SBC	LAB_B9,X		; subtract the stepper x delta low byte
	STA	LAB_B9,X		; save the ABS stepper x delta low byte
	TYA				; clear A
	SBC	LAB_BF,X		; subtract the stepper x delta high byte
	STA	LAB_BF,X		; save the ABS stepper x delta high byte

; compare the current delta with the maximum delta and overwrite it if greater

LAB_F524:
	CMP	LAB_90		; compare the delta high byte with the max delta high byte
	BCC	LAB_F538		; if less skip the delta copy

	BNE	LAB_F530		; if <> go do the delta copy

	LDA	LAB_B9,X		; else get the stepper x delta low byte
	CMP	LAB_8F		; compare the delta low byte with the max delta low byte
	BCC	LAB_F538		; if less skip the delta copy

; copy the current delta to the maximum delta

LAB_F530:
	LDA	LAB_B9,X		; get the stepper x delta low byte
	STA	LAB_8F		; save the maximum delta low byte
	LDA	LAB_BF,X		; get the stepper x delta high byte
	STA	LAB_90		; save the maximum delta high byte
LAB_F538:
	DEX				; decrement the stepper count/index
	BPL	LAB_F50A		; loop if more to do

; signed deltas have now been converted to directions and counts
;
; calculate the halfway value for the interpolation setup

	LDA	LAB_90		; get the maximum delta high byte
	STA	LAB_92		; save the move step count high byte
	LSR				; / 2
	TAY				; copy the maximum delta / 2 high byte
	LDA	LAB_8F		; get the maximum delta low byte
	STA	LAB_91		; save the move step count low byte
	ROR				; / 2

; set the interpolation totals to their mid points

	LDX	#$05			; set the stepper count/index
LAB_F548:
	STA	LAB_D1,X		; save the interpolation total low byte
	STY	LAB_D7,X		; save the interpolation total high byte
	DEX				; decrement the stepper count/index
	BPL	LAB_F548		; loop if more to do

	LDA	LAB_A6		; get the T1Ch value
	STA	LAB_A8		; save the timer 1 minimum value high byte
	LDA	LAB_A4		; get the acceleration control flag
	BEQ	LAB_F572		; if no acceleration control skip the setup

; setup for acceleration cotrol
;
; this sets up a $40 step acceleration/deceleration if there is room. if there is < $80
; steps in the entire move then it sets up a n\2 step acceleration/deceleration.
;
; if there is not enough room then the step rate will never reach its fastest value

	LDA	LAB_90		; get the maximum delta high byte
	LSR				; / 2
	BNE	LAB_F563		; if the maximum delta is >= 1024 go set acceleration step
					; count

	LDA	LAB_8F		; get the maximum delta low byte
	ROR				; / 2
	CMP	#$40			; compare it with the maximum step count
	BCC	LAB_F565		; if the maximum delta is < 128 go save the acceleration
					; step count

; set the acceleration step count to $40 steps

LAB_F563:
	LDA	#$40			; set the acceleration step count
LAB_F565:
	STA	LAB_A9		; save the acceleration step count
	CLC				; clear carry for add
	LDA	LAB_A8		; get the timer 1 minimum value high byte
	ADC	#$20			; + $2000, $40 steps of $0080
	BCC	LAB_F578		; if no overflow go set T1 to A,$00

; else the value overflowed so disable acceleration control for this move

	LDA	#$00			; clear A
	STA	LAB_A4		; clear the acceleration control flag
LAB_F572:
	STA	LAB_A9		; clear the acceleration step count
	STA	LAB_A5		; clear the T1Cl value
	BEQ	LAB_F582		; go set the mode to moving, branch always

; set T1 to A,$00

LAB_F578:
	LDY	#$00			; clear Y
	STY	LAB_A5		; clear the T1Cl value
	STY	LAB_7806		; clear VIA T1Ll
	JSR	LAB_F848		; set T1Ch and the button scan wait count
LAB_F582:
	LDA	#$01			; set the mode to moving
	STA	LAB_80		; save the mode byte
LAB_F586:
	LDA	LAB_81		; get the last button pressed
	CMP	#$07			; compare it with STOP
	BEQ	LAB_F59C		; if it was STOP go clear the mode and exit

	LDA	LAB_86		; get the line received flag
	BNE	LAB_F59C		; if a line has been received go clear the mode and exit

	LDA	LAB_80		; get the mode byte
	BNE	LAB_F586		; loop if a mode is still active

; move complete

	STA	LAB_A4		; clear the acceleration control flag
	RTS

; handle a positive movement delta

LAB_F597:
	INC	LAB_C5,X		; increment the stepper x direction low byte
	JMP	LAB_F524		; go compare the delta with the maximum delta

; move aborted, clear the mode and exit

LAB_F59C:
	STX	LAB_9F		; set the no increment flag, X = $FF
	INX				; clear X
	STX	LAB_80		; clear the mode byte
	STX	LAB_A4		; clear the acceleration control flag
	RTS


;************************************************************************************
;
; half step sequence
;
; the coil order is 4 2 3 1

LAB_F5A4:
	.byte	$01			; ...x
	.byte	$05			; .x.x
	.byte	$04			; .x..
	.byte	$06			; .xx.
	.byte	$02			; ..x.
	.byte	$0A			; x.x.
	.byte	$08			; x...
	.byte	$09			; x..x


;************************************************************************************
;
; NMI vector, triggered by the VIA T1

LAB_F5AC:
	BIT	LAB_7804		; read VIA T1Cl, clear the interrupt
	PHA				; save A
	TXA				; copy X
	PHA				; save X
	TYA				; copy Y
	PHA				; save Y
	LDA	LAB_80		; get the mode byte
	BMI	LAB_F5BB		; if TRAIN or MOVE go handle the pendant

	JMP	LAB_F63C		; else go do the positive modes

; scan the pendant stepper buttons

LAB_F5BB:
	LDA	#$50			; set IOIO IIII, column 4
	STA	LAB_7802		; save VIA DDRB

	LDA	LAB_7801		; read VIA DRA
	EOR	#$FF			; toggle all the bits
	ASL				; shift d7 to the carry, discard it
	ASL				; shift d6 to the carry, discard it

; read each button in turn. make the switch step byte 0 if the button is open or 1 if
; the button is closed

	LDY	#$00			; clear Y
	LDX	#$05			; set the switch count/index
LAB_F5CB:
	STY	LAB_B9,X		; clear the stepper x delta low byte
	ASL				; shift a button bit to the carry
	ROL	LAB_B9,X		; shift the bit to the stepper x delta low byte
	DEX				; decrement the switch count/index
	BPL	LAB_F5CB		; loop if more to do

; toggle the output for the other column

	LDA	#$60			; set IOOI IIII, column 5
	STA	LAB_7802		; save VIA DDRB

	LDA	LAB_7801		; read VIA DRA
	ASL				; shift d7 to the carry, discard it
	ASL				; shift d6 to the carry, discard it

; read each button in turn. make no change to the switch step byte if the button is open
; or decrement the switch step byte if the button is closed

	LDX	#$05			; set the switch count/index
LAB_F5DF:
	ASL				; shift a button bit to the carry
	BCS	LAB_F5E4		; if the button is open skip the decrement

	DEC	LAB_B9,X		; else decrement the stepper x delta low byte
LAB_F5E4:
	DEX				; decrement the switch count/index
	BPL	LAB_F5DF		; loop if more to do

; now convert from switch values to stepper values, first decouple the elbow and gripper
; movements

	SEC				; set carry for subtract
	LDA	LAB_BB		; get the elbow stepper delta low byte
	SBC	LAB_BE		; subtract the gripper step
	STA	LAB_BE		; save the gripper stepper delta low byte

; now convert wrist flex and wrist rotate into left and right wrist stepper values

	CLC				; clear carry for add
	LDA	LAB_BC		; get the wrist flex step
	TAX				; copy it
	ADC	LAB_BD		; add the wrist rotate step
	STA	LAB_BC		; save the right wrist stepper delta low byte

	SEC				; set carry for subtract
	TXA				; get the wrist flex step
	SBC	LAB_BD		; subtract the wrist rotate step
	STA	LAB_BD		; save the left wrist stepper delta low byte

; now change each stepper according to the step value

	LDX	#$05			; set the stepper count/index
LAB_F5FE:
	LDA	LAB_B9,X		; get the stepper x delta low byte
	BEQ	LAB_F636		; if zero go do the next stepper

	BMI	LAB_F617		; if negative go decrement the stepper count

; else it is positive so increment the stepper count

	LDY	LAB_80		; get the mode byte
	INY				; increment it
	BNE	LAB_F60E		; if MOVE mode go increment stepper x count

; else the arm is in MOVE mode so move the stepper without changing the count by changing
; the stepper phase offset

	INC	LAB_DD,X		; increment the stepper x phase offset
	JMP	LAB_F629		; go convert the stepper count to a drive pattern

; increment the stepper x count

LAB_F60E:
	INC	LAB_AD,X		; increment the stepper x count low byte
	BNE	LAB_F629		; if no rollover go convert the stepper count to a drive
					; pattern

	INC	LAB_B3,X		; increment the stepper x count high byte
	JMP	LAB_F629		; go convert the stepper count to a drive pattern

; the step value was negative so decrement the stepper count

LAB_F617:
	LDY	LAB_80		; get the mode byte
	INY				; increment it
	BNE	LAB_F621		; if MOVE mode go decrement the stepper x count

; else the arm is in MOVE mode so move the stepper without changing the count by changing
; the stepper phase offset

	DEC	LAB_DD,X		; decrement the stepper x phase offset
	JMP	LAB_F629		; go convert the stepper count to a drive pattern

; decrement the stepper x count

LAB_F621:
	LDA	LAB_AD,X		; get the stepper x count low byte
	BNE	LAB_F627		; if not zero skip the high byte decrement

	DEC	LAB_B3,X		; decrement the stepper x count high byte
LAB_F627:
	DEC	LAB_AD,X		; decrement the stepper x count low byte

; convert the stepper count and phase offset to a drive pattern

LAB_F629:
	LDA	LAB_AD,X		; get the stepper x count low byte
	CLC				; clear carry for add
	ADC	LAB_DD,X		; add the stepper x phase offset
	AND	#$07			; mask 0000 0xxx, 0 to 7 phase value
	TAY				; copy it to the index
	LDA	LAB_F5A4,Y		; get the half step drive pattern
	STA	LAB_E3,X		; save the stepper X drive pattern
LAB_F636:
	DEX				; decrement the stepper count/index
	BPL	LAB_F5FE		; loop if more to do

	JMP	LAB_F6C5		; go set the 74LS174 hex D-type flip flops from the stepper
					; drive patterns

; handle the positive modes

LAB_F63C:
	BEQ	LAB_F648		; if the mode is $00 go scan the buttons and exit

; mode is positive and non-zero

	LDX	LAB_91		; get the move step count low byte
	BNE	LAB_F64D		; if no rollunder go decrement it

	LDA	LAB_92		; get the move step count high byte
	BNE	LAB_F64B		; if not zero go decrement it

	STA	LAB_80		; else clear the mode byte, the move is done
LAB_F648:
	JMP	LAB_F6E9		; scan the buttons and exit

; mode is positive non-zero and step counts remain

LAB_F64B:
	DEC	LAB_92		; decrement the move step count high byte
LAB_F64D:
	DEC	LAB_91		; decrement the move step count low byte

	LDA	LAB_A4		; get the acceleration control flag
	BEQ	LAB_F68A		; if no acceleration control go calculate the next
					; interpolation point

; else calculate the new motor step time

	LDA	LAB_92		; get the move step count high byte
	BNE	LAB_F65B		; if > $FF then go test for acceleration

	CPX	LAB_A9		; compare the move step count low byte with the acceleration
					; step count
	BEQ	LAB_F675		; if there go decelerate

LAB_F65B:
	LDA	LAB_A6		; get the T1Ch value
	CMP	LAB_A8		; compare it with the timer 1 minimum value high byte
	BNE	LAB_F665		; if not there yet go accelerate

	LDA	LAB_A5		; get the T1Cl value
	BEQ	LAB_F68A		; if $00 skip the acceleration

; accelerate, subtract $0080 from the T1 value

LAB_F665:
	SEC				; set carry for subtract
	LDA	LAB_A5		; get the T1Cl value
	SBC	#$80			; - $0080 low byte
	STA	LAB_A5		; save the T1Cl value
	STA	LAB_7806		; save VIA T1Ll
	LDA	LAB_A6		; get the T1Ch value
	SBC	#$00			; - $0080 high byte
	BNE	LAB_F684		; go save the timer value, branch always

; decelerate, add $0080 to the T1 value

LAB_F675:
	DEC	LAB_A9		; decrement the acceleration step count so that next go
					; through will also match
	LDA	LAB_A5		; get the T1Cl value
	ADC	#$80-1		; + $0080 low byte, carry is set here
	STA	LAB_A5		; save the T1Cl value
	STA	LAB_7806		; save VIA T1Ll
	LDA	LAB_A6		; get the T1Ch value
	ADC	#$00			; + $0080 high byte
LAB_F684:
	STA	LAB_7807		; save VIA T1Lh
	JSR	LAB_F84B		; save the T1Ch value and set the button scan wait count

; calculate the next interpolation point and move the stepper if needed

LAB_F68A:
	LDX	#$05			; set the stepper count/index
LAB_F68C:
	LDA	LAB_D1,X		; get the interpolation total low byte
	SEC				; set carry for subtract
	SBC	LAB_B9,X		; subtract the stepper x delta low byte
	STA	LAB_D1,X		; save the interpolation total low byte
	LDA	LAB_D7,X		; get the interpolation total high byte
	SBC	LAB_BF,X		; subtract the stepper x delta high byte
	STA	LAB_D7,X		; save the interpolation total high byte
	BPL	LAB_F6C2		; if still positive go do the next stepper

; else the interpolation total has gone negative so reset it and move the stepper

	CLC				; clear carry for add
	LDA	LAB_D1,X		; get the interpolation total low byte
	ADC	LAB_8F		; add the maximum delta low byte
	STA	LAB_D1,X		; save the interpolation total low byte
	LDA	LAB_D7,X		; get the interpolation total high byte
	ADC	LAB_90		; add the maximum delta high byte
	STA	LAB_D7,X		; save the interpolation total high byte

; add the delta to the stepper position

	CLC				; clear carry for add
	LDA	LAB_AD,X		; get the stepper x count low byte
	ADC	LAB_C5,X		; add the stepper x direction low byte
	STA	LAB_AD,X		; save the stepper x count low byte
	TAY				; copy the stepper x count low byte
	LDA	LAB_B3,X		; get the stepper x count high byte
	ADC	LAB_CB,X		; add the stepper x direction high byte
	STA	LAB_B3,X		; save the stepper x count high byte

; convert the stepper position and phase offset to a drive pattern

	TYA				; copy the stepper x count low byte
	CLC				; clear carry for add
	ADC	LAB_DD,X		; add the stepper x phase offset
	AND	#$07			; mask 0000 0xxx, 0 to 7 phase value
	TAY				; copy it to the index
	LDA	LAB_F5A4,Y		; get the half step phase pattern
	STA	LAB_E3,X		; save the stepper X drive pattern
LAB_F6C2:
	DEX				; decrement the stepper count/index
	BPL	LAB_F68C		; loop if more to do

; set the 74LS174 hex D-type flip flops from the stepper drive pattern bytes

LAB_F6C5:
	LDA	LAB_E4		; get the stepper 1 drive pattern
	ASL				; shift the ..
	ASL				; .. stepper 1 ..
	ASL				; .. drive pattern ..
	ASL				; .. to b7-b4
	ORA	LAB_E3		; OR with the stepper 0 drive pattern
	STA	LAB_0800		; save 74LS174 hex D-type flip flop 0
	AND	#$F0			; clear the stepper 0 drive pattern
	ORA	LAB_E5		; OR with the stepper 2 drive pattern
	STA	LAB_0C00		; save 74LS174 hex D-type flip flop 1

	LDA	LAB_E7		; get the stepper 4 drive pattern
	ASL				; shift the ..
	ASL				; .. stepper 4 ..
	ASL				; .. drive pattern ..
	ASL				; .. to b7-b4
	ORA	LAB_E6		; OR with the stepper 3 drive pattern
	STA	LAB_1000		; save 74LS174 hex D-type flip flop 2
	AND	#$F0			; clear the stepper 3 drive pattern
	ORA	LAB_E8		; OR with the stepper 5 drive pattern
	STA	LAB_1400		; save 74LS174 hex D-type flip flop 3

; scan the buttons and exit

LAB_F6E9:
	DEC	LAB_89		; decrement the button scan wait counter
	BNE	LAB_F717		; if not counted out skip the button scan

	LDA	LAB_8A		; get the button scan wait count
	STA	LAB_89		; save the button scan wait counter

; scan the buttons, start with column 5

	LDA	#$60			; set IOOI IIII, column 5
	STA	LAB_7802		; save VIA DDRB

	LDA	LAB_7801		; read VIA DRA
	STA	LAB_A3		; save the pendant column 5 state
	ASL				; shift d7 to the carry, discard it

	LDX	#$0E			; set the switch count/index
LAB_F6FE:
	ASL				; shift a button bit to the carry
	BCC	LAB_F71D		; if the button is pressed go handle it

	DEX				; decrement the switch count/index
	CPX	#$07			; compare it with the end of the first loop
	BNE	LAB_F6FE		; loop if more to do

; done column 5, now do column 4

	LDA	#$50			; set IOIO IIII, column 4
	STA	LAB_7802		; save VIA DDRB

	LDA	LAB_7801		; read VIA DRA
	ASL				; shift d7 to the carry, discard it
LAB_F70F:
	ASL				; shift a button bit to the carry
	BCC	LAB_F71D		; if the button is pressed go handle the press

	DEX				; decrement the switch count/index
	BNE	LAB_F70F		; loop if more to do

; done column 4 and nothing pressed

	STX	LAB_83		; no button was pressed so clear the last held button
LAB_F717:
	PLA				; pull Y
	TAY				; restore Y
LAB_F719:
	PLA				; pull X
	TAX				; restore X
	PLA				; restore A
	RTI

; a button is pressed, the button index is in X

LAB_F71D:
	LDA	LAB_83		; get the last held button
	BNE	LAB_F717		; if there was a last held button restore Y, X and A and
					; return from interrupt

	LDA	LAB_80		; get the mode byte
	BPL	LAB_F731		; if not TRAIN or MOVE mode go save the button and exit

	CPX	#$07			; compare the button with MODE
	BEQ	LAB_F731		; if MODE go save the button and exit

	CPX	#$0E			; compare the button with REC
	BNE	LAB_F717		; if not REC go restore Y, X and A and return from the
					; interrupt

	CMP	#$F1			; compare the mode with TRAIN mode
	BNE	LAB_F717		; if not TRAIN go restore Y, X and A and return from the
					; interrupt

LAB_F731:
	STX	LAB_81		; save the last button pressed
	STX	LAB_82		; save the @READ last button pressed
	BPL	LAB_F717		; restore Y, X and A and return from the interrupt,
					; branch always


;************************************************************************************
;
; interpret a program step

LAB_F737:
	LDY	#$00			; clear the index
	LDA	LAB_9F		; get the no increment flag
	STY	LAB_9F		; clear the no increment flag
	BNE	LAB_F742		; if set skip the program step increment

	JSR	LAB_F1CF		; increment the the program step pointer
LAB_F742:
	LDA	(LAB_8D),Y		; get the step command byte
	CMP	#$07			; compare the command with the move with no acceleration
					; control command
	BEQ	LAB_F74E		; if it is the command go do it

	CMP	#$01			; compare the command with the move with acceleration
					; control command
	BNE	LAB_F77A		; if it isn't the command continue

; the command is move with acceleration control
;
;	$01,$ss,$0l,$1l,$2l,$3l,$4l,$5l,$0h,$1h,$2h,$3h,$4h,$5h

	STA	LAB_A4		; set the acceleration control flag

; the command is move with no acceleration control
;
;	$07,$ss,$0l,$1l,$2l,$3l,$4l,$5l,$0h,$1h,$2h,$3h,$4h,$5h


LAB_F74E:
	INY				; increment the index to the speed byte
	LDA	(LAB_8D),Y		; get the speed byte
	JSR	LAB_F848		; set T1Ch and the button scan wait count

; the positios in the commands are absolute couts so they have to have the current counts
; subtracted to tur them into deltas for each stepper

	LDY	#$07			; set the index to 5l
LAB_F756:
	SEC				; set carry for subtract
	LDA	(LAB_8D),Y		; get the absolute position low byte
	SBC	LAB_AD-2,Y		; subtract the stepper x count low byte
	STA	LAB_B9-2,Y		; save the stepper x delta low byte
	TYA				; copy the index
	PHP				; save the carry state
	CLC				; clear carry for add
	ADC	#$06			; add the offset to the high byte
	TAY				; copy the index back
	PLP				; restore the carry state
	LDA	(LAB_8D),Y		; get the absolute position high byte
	SBC	LAB_AD-2,Y		; subtract the stepper x count high byte
	STA	LAB_B9-2,Y		; save the stepper x delta high byte
	TYA				; copy the index
	SEC				; set carry for subtract
	SBC	#$07			; subtract the offset to the previous low byte
	TAY				; copy the index back
	CMP	#$01			; compare the idex with the miimum - 1
	BNE	LAB_F756		; loop if not all done

	JMP	LAB_F502		; setup the values for a move and return

; continue step interpretation

LAB_F77A:
	CMP	#$03			; compare the command with $03, JUMP command
	BNE	LAB_F7DE		; if the command isn't JUMP continue

; the command is JUMP

	INY				; increment the index to the condition code
	LDA	(LAB_8D),Y		; get the condition code
	CMP	#$08			; compare the condition code with 8, jump never
	BEQ	LAB_F7DD		; if it is jump never just exit

	BCS	LAB_F78D		; if > 8 go do an unconditional jump

; handle items 0 to 7

	TAX				; copy the item to the index
	LDA	LAB_1800,X		; read the 74LS251 data selector bit X
	BMI	LAB_F7DD		; if the bit = 1 just exit

; else do the jump

LAB_F78D:
	LDY	#$03			; set the index to the jump destination high byte
	LDA	(LAB_8D),Y		; get the jump destination high byte
	TAX				; copy the jump destination high byte
	DEY				; decrement the index to the jump destination low byte
	LDA	(LAB_8D),Y		; get the jump destination low byte
	STY	LAB_9F		; set the no increment flag


;************************************************************************************
;
; set the program step pointer to step XA

LAB_F797:
	STX	LAB_8E		; save the the program step pointer high byte
	ASL				; * 2 low byte
	ROL	LAB_8E		; * 2 the program step pointer high byte
	STA	LAB_8D		; save the the * 2 program step pointer low byte
	ASL				; * 4 low byte
	TAY				; copy the * 4 low byte
	LDA	LAB_8E		; get the the program step pointer high byte
	ROL				; * 4 high byte
	TAX				; copy the * 4 high byte
	TYA				; restore the * 4 low byte
	ADC	LAB_8D		; add the the * 2 program step pointer low byte
	STA	LAB_8D		; save the the * 6 program step pointer low byte
	TXA				; restore the * 4 high byte
	ADC	LAB_8E		; add the the * 2 program step pointer high byte
	STA	LAB_8E		; save the the * 6 program step pointer high byte
	TYA				; restore the * 4 low byte
	ASL				; * 8 low byte
	TAY				; copy the * 8 low byte
	TXA				; restore the * 4 high byte
	ROL				; * 8 high byte
	TAX				; copy the * 8 high byte
	TYA				; restore the * 8 low byte
	ADC	LAB_8D		; add the the * 6 program step pointer low byte
	TAY				; copy the * 14 low byte
	TXA				; restore the * 8 high byte
	ADC	LAB_8E		; add the the * 6 program step pointer high byte
	TAX				; copy the * 14 high byte
	TYA				; restore the * 14 low byte
	ADC	#<LAB_011A		; add the program memory start low byte
	STA	LAB_8D		; save the the program step pointer low byte
	TAY				; copy the the program step pointer low byte
	TXA				; restore the * 14 high byte
	ADC	#>LAB_011A		; add the program memory start high byte
	STA	LAB_8E		; save the the program step pointer high byte
	TAX				; copy the the program step pointer high byte
	CMP	#>LAB_07FE		; compare it with the 2K RAM end high byte
	BCC	LAB_F7DD		; if less just exit

	BNE	LAB_F7D3		; if greater go adjust the pointer to the ROM

	TYA				; restore the the program step pointer low byte
	CMP	#<LAB_07FE		; compare it with the 2K RAM end low byte
	BCC	LAB_F7DD		; if less just exit

OFF_F234	= LAB_FA33-LAB_07FE-1
					; the offset to the pre programmed program steps
LAB_F7D3:
	TYA				; restore the the program step pointer low byte
	ADC	#<OFF_F234		; add the offset to the pre programmed program steps low
					; byte
	STA	LAB_8D		; save the the program step pointer low byte
	TXA				; restore the the program step pointer high byte
	ADC	#>OFF_F234		; add the offset to the pre programmed program steps high
					; byte
	STA	LAB_8E		; save the the program step pointer high byte
LAB_F7DD:
	RTS


;************************************************************************************
;
; continue step interpretation

LAB_F7DE:
	CMP	#$02			; compare the command with $02, GRIP command
	BNE	LAB_F7EB		; if the command isn't GRIP continue

; the command is GRIP

	INY				; increment the index to the speed byte
	LDA	(LAB_8D),Y		; get the speed byte
	JSR	LAB_F848		; set T1Ch and the button scan wait count
	JMP	LAB_F96D		; close the gripper and $20 steps more and return

; continue step interpretation

LAB_F7EB:
	CMP	#$05			; compare the command with $05, OUT command
	BNE	LAB_F7F9		; if the command isn't OUT continue

; the command is OUT

	INY				; increment the index to the item number
	LDA	(LAB_8D),Y		; read the item number
	TAX				; copy the item number
	INY				; increment the index to the item state
	LDA	(LAB_8D),Y		; read the item state
	JMP	LAB_F1B4		; set item X to state A and return

; continue step interpretation

LAB_F7F9:
	CMP	#$04			; compare the command with $04, PAUSE command
	BNE	LAB_F835		; if the command isn't PAUSE continue

; the command is PAUSE

	STY	LAB_91		; clear the move step count low byte
	STY	LAB_92		; clear the move step count high byte
	STY	LAB_95		; clear the pause count high byte
	INY				; increment the index to the pause value
	LDA	(LAB_8D),Y		; get the pause value
	LSR				; / 2 * 256 high byte
	ROR	LAB_94		; / 2 * 256 low byte
	LSR				; / 4 * 256 high byte
	ROR	LAB_94		; / 4 * 256 low byte
	LSR				; / 8 * 256 high byte
	ROR	LAB_94		; / 8 * 256 low byte
	STA	LAB_95		; save the pause count high byte
	LDA	#$E0			; set 1843200 / $E002 / 8 * 256 * n ~ n seconds
	JSR	LAB_F848		; set T1Ch and the button scan wait count
LAB_F816:
	LDA	#$01			; set mode $01, moving
	STA	LAB_80		; save the mode byte
LAB_F81A:
	LDA	LAB_80		; get the mode byte
	BNE	LAB_F81A		; wait if not done

	LDA	LAB_94		; get the pause count low byte
	BNE	LAB_F828		; if no rollunder skip the high byte decrement

	LDA	LAB_95		; get the pause count high byte
	BEQ	LAB_F834		; if it is zero then the time is up so just exit

	DEC	LAB_95		; else decrement the pause count high byte
LAB_F828:
	DEC	LAB_94		; decrement the pause count low byte
	LDA	LAB_81		; get the last button pressed
	CMP	#$07			; compare it with MODE
	BEQ	LAB_F834		; if it is MODE go exit the pause

	LDA	LAB_86		; get the line received flag
	BEQ	LAB_F816		; if no line has been received go continue the pause

LAB_F834:
	RTS

; continue step interpretation

LAB_F835:
	CMP	#$06			; compare the command with the text command
	BNE	LAB_F834		; if the command isn't the text command just exit

; the command is TEXT
;
;	$06,$tt,$tt,$tt,$tt,$tt,$tt,$tt,$tt,$tt,$tt,$tt,$tt,$00

LAB_F839:
	INY				; increment the index to the next text character
	STY	LAB_9B		; save the index
	LDA	(LAB_8D),Y		; get the text character
	BEQ	LAB_F834		; if it is null just exit

	TAX				; copy the character to X
	JSR	LAB_F8FC		; send ACIA A byte with delay
	LDY	LAB_9B		; restore the index
	BNE	LAB_F839		; go get another character, branch always


;************************************************************************************
;
; set T1Ch and the button scan wait count

LAB_F848:
	STA	LAB_7805		; save VIA T1Ch


;************************************************************************************
;
; save the T1Ch value and set the button scan wait count
;
; the faster T1 runs the greater the button scan wait count so button scanning is
; mostly never too fast or ever too slow

LAB_F84B:
	STA	LAB_A6		; save the T1Ch value
	EOR	#$FF			; toggle all the bits
	LSR				; 0xxx xxxx
	LSR				; 00xx xxxx
	LSR				; 000x xxxx
	LSR				; 0000 xxxx
	LSR				; 0000 0xxx
	ORA	#$01			; 0000 0xx1
	STA	LAB_8A		; save the button scan wait count
	RTS


;************************************************************************************
;
; input a signed decimal value

LAB_F859:
	LDA	#$00			; clear A
	STA	LAB_97		; clear the result low byte
	STA	LAB_98		; clear the result high byte
	STA	LAB_99		; clear the negative flag
	STA	LAB_9C		; clear the digit received flag
LAB_F863:
	JSR	LAB_F9B3		; get a byte from the serial buffer
	CMP	LAB_88		; compare it with the recognition character
	BEQ	LAB_F88D		; if it is the recognition character return bad syntax
					; and exit

	AND	#$7F			; mask 0xxx xxxx, 7 bit ASCII
	CMP	#$0D			; compare the byte with [CR]
	BEQ	LAB_F8A3		; if it is [CR] go decrement the buffer poiter and return
					; the value

	CMP	#','			; compare the byte with ","
	BEQ	LAB_F8AB		; if it is "," go return the value

	CMP	#'-'			; compare the byte with "-"
	BNE	LAB_F87C		; if isn't "-" go try "0" to "9"

	STA	LAB_99		; else set the negative flag
	BEQ	LAB_F863		; go get the next character, branch always

LAB_F87C:
	CMP	#'0'			; compare the byte with "0"
	BCC	LAB_F863		; if it is < "0" go get another character

	CMP	#'9'+1		; compare the byte with "9" + 1
	BCS	LAB_F863		; if it is > "9" go get another character

	BCC	LAB_F8C7		; else go add its value to the total, branch always

; the digit loop, once the first digit is encoutered all further character gets are
; doe from here

LAB_F886:
	JSR	LAB_F9B3		; get a byte from the serial buffer
	CMP	LAB_88		; compare it with the recognition character
	BNE	LAB_F895		; if it isn't the recognition character go try "0" to "9"

; return bad syntax and exit

LAB_F88D:
	LDX	#'0'			; set "0", bad syntax
	JSR	LAB_F481		; respond CHR$(X)
	JMP	LAB_F0BE		; return to the start of the main loop

; the character wasn't the recognition character

LAB_F895:
	AND	#$7F			; mask 0xxx xxxx, 7 bit ASCII
	CMP	#','			; compare the byte with ","
	BEQ	LAB_F8AB		; if it is "," go return the value

	CMP	#' '			; compare the byte with " "
	BEQ	LAB_F8AB		; if it is " " go return the value

	CMP	#$0D			; compare the byte with [CR]
	BNE	LAB_F8BF		; if isn't [CR] go try "0" to "9"

LAB_F8A3:
	LDA	LAB_8C		; get the Rx buffer read index
	SBC	#$01			; decrement it
	AND	#$7F			; mask 0xxx xxxx, the buffer extent
	STA	LAB_8C		; save the Rx buffer read index
LAB_F8AB:
	LDA	LAB_99		; get the negative flag
	BEQ	LAB_F8BA		; if not set go return the positive result

; calculate and return the negative result

	SEC				; set carry for subtract
	LDA	#$00			; clear A
	SBC	LAB_97		; subtract the result low byte
	TAY				; copy the result low byte
	LDA	#$00			; clear A
	SBC	LAB_98		; subtract the result high byte
	RTS

; return the positive result

LAB_F8BA:
	LDY	LAB_97		; get the result low byte
	LDA	LAB_98		; get the result high byte
	RTS

; test for "0" to "9" and add it to the result if it is

LAB_F8BF:
	CMP	#'0'			; compare the byte with "0"
	BCC	LAB_F886		; if it is < "0" go get another character

	CMP	#'9'+1		; compare the byte with "9" + 1
	BCS	LAB_F886		; if it is > "9" go get another character

; the character was "0" to "9" so mulitply the value by 10 and add it

LAB_F8C7:
	STA	LAB_9C		; save the digit received flag
	SBC	#'0'-1		; subtract "0" - 1, carry clear here

	ASL	LAB_97		; * 2 result low byte
	ROL	LAB_98		; * 2 result high byte
	ADC	LAB_97		; digit + * 2 result low byte
	TAY				; copy it
	LDA	LAB_98		; get the * 2 result high byte
	ADC	#$00			; add any carry
	TAX				; copy it
	ASL	LAB_97		; * 4 result low byte
	ROL	LAB_98		; * 4 result high byte
	ASL	LAB_97		; * 8 result low byte
	ROL	LAB_98		; * 8 result high byte
	CLC				; clear carry for add
	TYA				; copy the digit + * 2 result low byte
	ADC	LAB_97		; add the * 8 result low byte
	STA	LAB_97		; save the digit + * 10 result low byte
	TXA				; copy the digit + * 2 result high byte
	ADC	LAB_98		; add the * 8 result high byte
	STA	LAB_98		; save the digit + * 10 result high byte
	JMP	LAB_F886		; go get another character


;************************************************************************************
;
; input a signed decimal value, set T1Ch and the button scan wait count

LAB_F8ED:
	JSR	LAB_F859		; input a signed decimal value
	TYA				; copy the input value low byte
	LDY	LAB_9C		; get the digit received flag
	BNE	LAB_F8F7		; if there were digits skip the default set

	LDA	#$E5			; if no value returned use $1A
LAB_F8F7:
	EOR	#$FF			; toggle all the bits
	JMP	LAB_F848		; set T1Ch and the button scan wait count


;************************************************************************************
;
; send ACIA A byte with delay

LAB_F8FC:
	LDY	LAB_A2		; get the Tx character delay
	INY				; + 1
	TXA				; copy the character
	LDX	#$01			; set the delay inner loop start
LAB_F902:
	DEX				; decrement the inner loop count
	BNE	LAB_F902		; loop if more to do

	DEY				; decrement the character delay
	BNE	LAB_F902		; loop if more to do

	TAX				; copy the Tx data byte


;************************************************************************************
;
; send ACIA A byte

LAB_F909:
	LDA	#$02			; mask 0000 00x0, ACIA A Tx status bit
LAB_F90B:
	BIT	LAB_4800		; test the ACIA A Tx status bit
	BEQ	LAB_F90B		; loop while the Tx buffer is full

	STX	LAB_4801		; save the ACIA A Tx data byte
	RTS


;************************************************************************************
;
; get a numeric value from the buttons

LAB_F914:
	LDX	#$01			; set the LED to off
	STX	LAB_1C00		; save the MODE LED

	DEX				; clear X
	STX	LAB_84		; clear the numeric value low byte
	STX	LAB_85		; clear the numeric value high byte
LAB_F91E:
	LDA	LAB_83		; get the last held button
	BNE	LAB_F91E		; if a button is held go wait some more

	LDA	#$40			; set the ENTER LED on
	STA	LAB_7800		; save VIA DRB

	JSR	LAB_F99E		; wait for or return a button
	TAX				; copy the button

	LDA	#$00			; set the ENTER LED off
	STA	LAB_7800		; save VIA DRB

	LDA	LAB_86		; get the line received flag
	BNE	LAB_F948		; if a line has been received go return the value

	LDA	LAB_F01E-1,X	; get the button digit value
	BPL	LAB_F94B		; if it was a digit go handle it

	CMP	#$FE			; compare the button with REC
	BEQ	LAB_F914		; if the button was REC go clear the value

	CMP	#$FB			; compare the button with MODE
	BEQ	LAB_F948		; if the button was MODE go return the value

	LDA	#$40			; set the ENTER LED on
	STA	LAB_7800		; save VIA DRB
	BNE	LAB_F91E		; go wait for another button, branch always

LAB_F948:
	LDA	LAB_84		; get the numeric value low byte
	RTS

; the button was a 0 to 9 digit, multiply the value by 10 and add the button

LAB_F94B:
	ASL	LAB_84		; * 2 value low byte
	ROL	LAB_85		; * 2 value high byte
	ADC	LAB_84		; add the * 2 value low byte
	TAY				; copy the * 2 value + button low byte
	LDA	LAB_85		; get the value high byte
	ADC	#$00			; add any carry
	TAX				; copy the * 2 value + button high byte
	ASL	LAB_84		; * 4 value low byte
	ROL	LAB_85		; * 4 value high byte
	ASL	LAB_84		; * 8 value low byte
	ROL	LAB_85		; * 8 value high byte
	CLC				; clear carry for add
	TYA				; get the result low byte
	ADC	LAB_84		; add the * 8 value low byte
	STA	LAB_84		; save the * 10 value + button low byte
	TXA				; get the result high byte
	ADC	LAB_85		; add the * 8 value high byte
	STA	LAB_85		; save the * 10 value + button high byte
	JMP	LAB_F91E		; go wait for another button


;************************************************************************************
;
; close the gripper and $20 steps more

LAB_F96D:
	LDA	#$20			; set the number of steps to go past the switch point


;************************************************************************************
;
; close the gripper and A steps more

LAB_F96F:
	STA	LAB_96		; save the number of steps beyond the switch point
	BIT	LAB_1800		; read the 74LS251 data selector bit 0, gripper switch
	BMI	LAB_F97F		; if the gripper switch is open, the gripper is closed,
					; just exit

LAB_F976:
	BIT	LAB_1800		; read the 74LS251 data selector bit 0, gripper switch
	BPL	LAB_F980		; if it is closed, the gripper is open, go close the
					; gripper one step

					; else the gripper is closed, the switch is open so ..
	DEC	LAB_96		; decrement the number of steps beyond the switch point
	BNE	LAB_F980		; if it isn't zero go close the gripper one step

LAB_F97F:
	RTS

LAB_F980:
	LDX	#$0B			; set the byte count/index
	LDA	#$00			; clear A
LAB_F984:
	STA	LAB_B9,X		; save the stepper x delta byte
	DEX				; decrement the byte count/index
	BPL	LAB_F984		; loop if more to do

	STX	LAB_BE		; save the gripper stepper delta low byte
	STX	LAB_C4		; save the gripper stepper delta high byte
	JSR	LAB_F502		; setup the values for a move
	CMP	#$07			; compare the byte returned with STOP
	BNE	LAB_F976		; if it isn't STOP go close the gripper some more

	RTS


;************************************************************************************
;
; reset the program step pointer

LAB_F995:
	LDA	#<LAB_011A		; set the program memory start low byte
	STA	LAB_8D		; save the the program step pointer low byte
	LDA	#>LAB_011A		; set the program memory start high byte
	STA	LAB_8E		; save the the program step pointer high byte
	RTS


;************************************************************************************
;
; wait for or return a button
;
; this waits until either a button is retured or a line is received over the serial
; port

LAB_F99E:
	JSR	LAB_F9A8		; get a button
	BNE	LAB_F9B2		; if a button was pressed just return it

	LDX	LAB_86		; get the line received flag
	BEQ	LAB_F99E		; if no line has been received go get a button again

	RTS


;************************************************************************************
;
; get a button

LAB_F9A8:
	LDX	#$00			; clear X
	LDA	LAB_81		; get the last button pressed
	BEQ	LAB_F9B2		; if null just exit

	STA	LAB_83		; save the last held button
	STX	LAB_81		; clear the last button pressed
LAB_F9B2:
	RTS


;************************************************************************************
;
; get a byte from the serial buffer

LAB_F9B3:
	LDX	LAB_8C		; get the Rx buffer read index
	LDA	LAB_00,X		; get an Rx buffer byte
	INX				; increment the Rx buffer read index
	BPL	LAB_F9BC		; if < $80 continue

	LDX	#$00			; else reset the index to the start
LAB_F9BC:
	STX	LAB_8C		; save the Rx buffer read index
LAB_F9BE:
	RTS


;************************************************************************************
;
; output AY as a signed, leading zero supressed, integer

LAB_F9BF:
	LDX	#$00			; clear X
	STX	LAB_9C		; clear the digits output flag
	STY	LAB_97		; save the word low byte
	STA	LAB_98		; save the word high byte
	TAY				; copy the word high byte
	BPL	LAB_F9DA		; if it is positive skip the sign and negate

	TXA				; clear A
	SEC				; set carry for subtract
	SBC	LAB_97		; subtract the word low byte
	STA	LAB_97		; save the word low byte
	TXA				; clear A
	SBC	LAB_98		; subtract the word high byte
	STA	LAB_98		; save the word high byte
	LDX	#'-'			; set "-"
	JSR	LAB_F8FC		; send ACIA A byte with delay
LAB_F9DA:
	LDY	#<$2710		; set 10,000 low byte
	LDA	#>$2710		; set 10,000 high byte
	JSR	LAB_FA03		; output the AY decimal digit
	LDY	#<$03E8		; set 1,000 low byte
	LDA	#>$03E8		; set 1,000 high byte
	JSR	LAB_FA03		; output the AY decimal digit
	LDY	#<$0064		; set 100 low byte
	LDA	#>$0064		; set 100 high byte
	JSR	LAB_FA03		; output the AY decimal digit
	LDY	#<$000A		; set 10 low byte
	LDA	#>$000A		; set 10 high byte
	JSR	LAB_FA03		; output the AY decimal digit
	LDY	#<$0001		; set 1 low byte
	LDA	#>$0001		; set 1 high byte
	JSR	LAB_FA03		; output the AY decimal digit
	LDA	LAB_9C		; get the digits output flag
	BNE	LAB_F9BE		; if digits output just exit

					; else there were no digits output and X now holds a "0"
					; character
	BEQ	LAB_FA30		; go send ACIA A byte with delay and return,
					; branch always


;************************************************************************************
;
; output the AY decimal digit

LAB_FA03:
	LDX	#'0'			; set the digit
	STY	LAB_9D		; save the digit value low byte
	STA	LAB_9E		; save the digit value high byte
LAB_FA09:
	SEC				; set carry for subtract
	LDA	LAB_97		; get the word low byte
	SBC	LAB_9D		; subtract the digit value low byte
	STA	LAB_97		; save the word low byte
	LDA	LAB_98		; get the word high byte
	SBC	LAB_9E		; subtract the digit value high byte
	STA	LAB_98		; save the word high byte
	BMI	LAB_FA1B		; if negative go adjust the value and output the digit

	INX				; else increment the digit
	BNE	LAB_FA09		; go do another subtract, branch always

; sutracted one too far so add it back

LAB_FA1B:
	TYA				; copy the digit value low byte
	ADC	LAB_97		; add the word low byte
	STA	LAB_97		; save the word low byte
	LDA	LAB_98		; get the word high byte
	ADC	LAB_9E		; add the digit value high byte
	STA	LAB_98		; save the word high byte
	CPX	#'0'			; compare the digit with "0"
	BNE	LAB_FA2E		; if not "0" just go output it

	LDA	LAB_9C		; get the digits output flag
	BEQ	LAB_F9BE		; if no digits output just exit

LAB_FA2E:
	STX	LAB_9C		; set the digits output flag
LAB_FA30:
	JMP	LAB_F8FC		; send ACIA A byte with delay and return


;************************************************************************************
;
; program step commands

; $xx	= do't care
; $nl	= stepper n low byte
; $nh	= stepper n high byte

; MOVE command, no acceleration control
;
;	07 = command byte
;	ss = speed value

;	$07,$ss,$0l,$1l,$2l,$3l,$4l,$5l,$0h,$1h,$2h,$3h,$4h,$5h

; TEXT command
;
;	06 = command byte
;	tt = ASCII character or null byte

;	$06,$tt,$tt,$tt,$tt,$tt,$tt,$tt,$tt,$tt,$tt,$tt,$tt,$00

; OUT command
;
;	05 = command byte
;	ii = item number
;	nn = 1 = on
;	   = 0 = off

;	$05,$ii,$nn,$xx,$xx,$xx,$xx,$xx,$xx,$xx,$xx,$xx,$xx,$xx

; PAUSE command
;
;	04 = command byte
;	pp = pause value

;	$04,$pp,$xx,$xx,$xx,$xx,$xx,$xx,$xx,$xx,$xx,$xx,$xx,$xx

; JUMP command
;
;	03 = command byte
;	cc = condition code
;	     $00 to $07 = 74LS251 input bits
;	     $08        = never
;	     $09        = always
;	ll = jump destination low byte
;	hh = jump destination high byte

;	$03,$cc,$ll,$hh,$xx,$xx,$xx,$xx,$xx,$xx,$xx,$xx,$xx,$xx

; GRIP command
;
;	02 = command byte
;	ss = speed value

;	$02,$ss,$xx,$xx,$xx,$xx,$xx,$xx,$xx,$xx,$xx,$xx,$xx,$xx

; MOVE command, with acceleration control
;
;	01 = command byte
;	ss = speed value

;	$01,$ss,$0l,$1l,$2l,$3l,$4l,$5l,$0h,$1h,$2h,$3h,$4h,$5h

; NULL command
;
;	00 = command byte

;	$00,$xx,$xx,$xx,$xx,$xx,$xx,$xx,$xx,$xx,$xx,$xx,$xx,$xx


;************************************************************************************
;
; pre programmed steps

LAB_FA33:
	.byte	$01,$10,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00	; 126
	.byte	$01,$11,$00,$37,$C6,$00,$00,$C6,$00,$FF,$FF,$00,$00,$FF
	.byte	$01,$10,$04,$EB,$BB,$00,$00,$BB,$F9,$FC,$FE,$00,$00,$FE
	.byte	$01,$10,$04,$B9,$4C,$00,$00,$4C,$F9,$02,$FC,$00,$00,$FC
	.byte	$01,$11,$04,$31,$C5,$1B,$E9,$C5,$F9,$FB,$F7,$FA,$FF,$F7	; 130
	.byte	$01,$0C,$04,$31,$C5,$1B,$E9,$5C,$F9,$FB,$F7,$FA,$FF,$FB
	.byte	$02,$0E,$A0,$B3,$A0,$A0,$C5,$B1,$A0,$B7,$A0,$A0,$A0,$A0
	.byte	$04,$02,$C1,$A0,$A0,$B7,$A0,$A0,$AC,$B9,$A0,$A0,$A0,$A0
	.byte	$01,$10,$04,$85,$48,$69,$37,$30,$F9,$FD,$F9,$F9,$FF,$F9
	.byte	$01,$0E,$EC,$96,$45,$04,$F4,$2D,$06,$02,$FC,$03,$FC,$FC
	.byte	$01,$11,$EC,$33,$E5,$04,$F4,$CD,$06,$FE,$FE,$03,$FC,$FE
	.byte	$01,$10,$00,$33,$8A,$0C,$EC,$72,$00,$FE,$FF,$00,$FF,$FF
	.byte	$01,$10,$00,$19,$C5,$0C,$EC,$AD,$00,$FF,$FF,$00,$FF,$FF
	.byte	$01,$32,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
	.byte	$01,$11,$00,$00,$00,$00,$00,$76,$00,$FF,$00,$00,$00,$04	; 140
	.byte	$01,$10,$00,$97,$23,$AD,$AD,$99,$00,$00,$FE,$00,$00,$02
	.byte	$01,$0E,$00,$97,$23,$AD,$AD,$FD,$00,$00,$FE,$00,$00,$FF
	.byte	$03,$00,$A7,$00,$A0,$C4,$B1,$A0,$BF,$A0,$80,$A0,$C6,$B1
	.byte	$01,$17,$00,$6E,$23,$AD,$AD,$FD,$00,$FF,$FE,$00,$00,$FF
	.byte	$01,$10,$00,$06,$02,$26,$41,$DC,$00,$FE,$FF,$02,$FC,$00
	.byte	$01,$11,$00,$1F,$C1,$3D,$53,$96,$00,$00,$FF,$02,$FC,$01
	.byte	$01,$0C,$00,$1F,$C1,$3D,$53,$48,$00,$00,$FF,$02,$FC,$02
	.byte	$01,$13,$00,$80,$CE,$F0,$06,$55,$00,$FF,$FE,$02,$FD,$01
	.byte	$01,$0E,$00,$80,$CE,$F0,$06,$BA,$00,$FF,$FE,$02,$FD,$00
	.byte	$01,$11,$00,$A6,$05,$F0,$06,$A1,$00,$FE,$FE,$02,$FD,$FF	; 150
	.byte	$01,$13,$C5,$12,$F8,$8E,$72,$D7,$FD,$00,$FD,$02,$FD,$FF
	.byte	$07,$1C,$29,$EB,$1E,$A4,$5C,$FD,$FE,$FF,$FE,$02,$FD,$FF
	.byte	$07,$10,$96,$CE,$3C,$BB,$45,$1B,$FE,$FF,$FE,$02,$FD,$00
	.byte	$07,$10,$0A,$BA,$52,$D4,$2C,$31,$FF,$FF,$FE,$02,$FD,$00
	.byte	$07,$10,$83,$AE,$60,$EF,$11,$3F,$FF,$FF,$FE,$02,$FD,$00
	.byte	$07,$10,$00,$AB,$64,$0A,$F6,$43,$00,$FF,$FE,$03,$FC,$00
	.byte	$07,$10,$7D,$AE,$60,$25,$DB,$3F,$00,$FF,$FE,$03,$FC,$00
	.byte	$07,$10,$F6,$BA,$52,$40,$C0,$31,$00,$FF,$FE,$03,$FC,$00
	.byte	$07,$10,$6A,$CE,$3C,$59,$A7,$1B,$01,$FF,$FE,$03,$FC,$00
	.byte	$07,$10,$D7,$EB,$1C,$70,$90,$FD,$01,$FF,$FE,$03,$FC,$FF	; 160
	.byte	$07,$1C,$3B,$12,$F8,$86,$7A,$D7,$02,$00,$FD,$03,$FC,$FF
	.byte	$01,$13,$00,$66,$F0,$73,$83,$DC,$00,$FE,$FD,$01,$FE,$FF
	.byte	$01,$10,$00,$86,$BE,$73,$83,$AA,$00,$FF,$FE,$01,$FE,$00
	.byte	$01,$0C,$00,$86,$BE,$73,$83,$07,$00,$FF,$FE,$01,$FE,$02
	.byte	$01,$13,$00,$8D,$60,$73,$83,$A9,$00,$FC,$FD,$01,$FE,$00
	.byte	$03,$09,$7E,$00,$BA,$A0,$A0,$A0,$CF,$A0,$A0,$A0,$B0,$A4
	.byte	$01,$11,$00,$46,$D2,$78,$78,$3F,$00,$FE,$FC,$FE,$FE,$FD
	.byte	$01,$13,$00,$46,$D2,$13,$13,$3F,$00,$FE,$FC,$FE,$FE,$FD
	.byte	$01,$10,$00,$46,$D2,$A8,$A8,$3F,$00,$FE,$FC,$FE,$FE,$FD
	.byte	$01,$11,$00,$46,$D2,$17,$17,$3F,$00,$FE,$FC,$FE,$FE,$FD	; 170
	.byte	$01,$10,$00,$46,$D2,$95,$95,$3F,$00,$FE,$FC,$FE,$FE,$FD
	.byte	$01,$13,$00,$67,$0C,$00,$00,$0C,$00,$FD,$FE,$00,$00,$FE
	.byte	$03,$09,$7E,$00,$A0,$A0,$A0,$B6,$80,$A0,$80,$A0,$A0,$A0
	.byte	$01,$10,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
	.byte	$04,$04,$A0,$C4,$A0,$B8,$BA,$A0,$A0,$A0,$A0,$C5,$A0,$A0
	.byte	$01,$13,$2B,$18,$1D,$A9,$57,$18,$FE,$FE,$FE,$FF,$00,$02
	.byte	$01,$10,$6C,$C1,$9A,$A9,$57,$44,$FE,$FF,$FE,$FF,$00,$01
	.byte	$01,$0E,$6C,$C1,$9A,$A9,$57,$4C,$FE,$FF,$FE,$FF,$00,$00
	.byte	$03,$00,$C0,$00,$B3,$A0,$C6,$A4,$A0,$C4,$A0,$A0,$A0,$C5
	.byte	$01,$17,$00,$6F,$D4,$09,$03,$86,$00,$FD,$FD,$00,$00,$FF	; 180
	.byte	$01,$10,$00,$95,$C5,$15,$EB,$77,$00,$FF,$FE,$00,$FF,$00
	.byte	$01,$0C,$00,$95,$C5,$15,$EB,$54,$00,$FF,$FE,$00,$FF,$02
	.byte	$01,$13,$00,$6F,$D4,$09,$03,$86,$00,$FD,$FD,$00,$00,$FF
	.byte	$01,$10,$89,$A7,$85,$57,$A9,$63,$01,$FF,$FE,$00,$FF,$01
	.byte	$01,$0E,$89,$CA,$94,$4E,$B2,$A7,$01,$FF,$FE,$00,$FF,$FF
	.byte	$03,$00,$BE,$00,$C6,$A0,$A0,$A0,$A0,$A0,$C4,$A0,$9D,$D0	; 186
	.byte	$01,$17,$00,$2F,$C6,$00,$00,$DE,$00,$FD,$FD,$00,$00,$FE
	.byte	$01,$10,$00,$CC,$86,$F6,$05,$99,$00,$FE,$FE,$FF,$00,$FF
	.byte	$01,$0C,$00,$CC,$86,$F6,$05,$CE,$00,$FE,$FE,$FF,$00,$00
	.byte	$01,$13,$00,$9D,$B6,$00,$00,$06,$00,$FC,$FE,$00,$00,$02	; 190
	.byte	$03,$09,$AE,$00,$A0,$BA,$A0,$A0,$B3,$A0,$A0,$B2,$B0,$B7
	.byte	$01,$13,$2B,$18,$1D,$A9,$57,$18,$FE,$FE,$FE,$FF,$00,$02
	.byte	$01,$11,$64,$D1,$ED,$57,$A9,$C6,$01,$FD,$FD,$00,$FF,$00
	.byte	$01,$10,$91,$A7,$85,$63,$9D,$F4,$01,$FF,$FE,$00,$FF,$01
	.byte	$01,$0E,$91,$A7,$85,$63,$9D,$50,$01,$FF,$FE,$00,$FF,$00
	.byte	$03,$00,$BE,$00,$A0,$A0,$A0,$A0,$B6,$AC,$A0,$B7,$A0,$A0
	.byte	$01,$17,$00,$6F,$D4,$09,$03,$86,$00,$FD,$FD,$00,$00,$FF
	.byte	$01,$10,$00,$95,$C5,$00,$00,$77,$00,$FF,$FE,$00,$00,$00
	.byte	$01,$0C,$00,$95,$C5,$00,$00,$76,$00,$FF,$FE,$00,$00,$01
	.byte	$01,$13,$F5,$28,$A0,$06,$FA,$DB,$FF,$FD,$FD,$00,$FF,$00	; 200
	.byte	$01,$10,$4E,$C1,$9A,$A9,$57,$44,$FE,$FF,$FE,$FF,$00,$01
	.byte	$01,$0E,$4E,$C1,$9A,$A9,$57,$AD,$FE,$FF,$FE,$FF,$00,$FF
	.byte	$03,$09,$BA,$00	; step 203, unconditional JUMP to step 186

	.byte	$AC,$A0,$B1,$C4,$B4,$A0,$B1,$A0,$C6,$B5
	.byte	$B4,$A0,$B4,$A0,$A0,$BA,$A0,$B8,$A0,$C5,$A0,$A0,$A0,$A0
	.byte	$BA,$B1,$D3,$B4,$B6,$A0,$B7,$A0,$A0,$A0,$A0,$A0,$B4,$A0
	.byte	$BC,$A0,$BF,$A0,$A0,$B2,$A0,$BE,$A0,$AE,$A0,$C2,$B0,$A0
	.byte	$B5,$A4,$A0,$A0,$A0,$A0,$A0,$B1,$B1,$C6,$A0,$B6,$A0,$A0
	.byte	$A0,$C1,$81,$A0,$BF,$A0,$A0,$A0,$A0,$B4,$A0,$A0,$A0,$A0
	.byte	$BA,$A0,$C2,$A0,$C5,$A0,$A0,$A0,$B1,$BA,$B1,$D3,$A0,$A0
	.byte	$A0,$A0,$A0,$D2,$A0,$A0,$C1,$B4,$A0,$BF,$A0,$80,$A0,$A0
	.byte	$B3,$A0,$BE,$A0,$A0,$D5,$A0,$B1,$D0,$C6,$A0,$AD,$D0,$A0
	.byte	$A0,$A0,$B7,$A0,$C6,$A0,$A0,$A0,$A0,$A0,$A0,$81,$D9,$C6
	.byte	$A0,$A0,$A0,$A0,$B4,$A0,$C6,$A0,$B0,$A0,$A0,$C4,$B2,$A0
	.byte	$BF,$A0,$80,$A0,$A0,$B0,$A0,$C6,$A4,$A0,$CD,$A0,$A0,$AE
	.byte	$C6,$A0,$C6,$D4,$A0,$BB,$D6,$A0,$A0,$A0,$A0,$84,$A0,$A0
	.byte	$A0,$A0,$80,$A0,$BF,$A0,$A0,$BA,$B3,$C4,$CD,$C5,$A0,$CA
	.byte	$C6,$A0,$BA,$A0,$A0,$B4,$A0,$A0,$B4,$A0,$A0,$A0,$B1,$A0
	.byte	$B4,$A0,$A0,$A0,$A0,$A0,$C6,$B2,$A0,$BF,$A0,$80,$A0,$AE
	.byte	$B1,$A0,$A0,$A0,$AC,$C4,$A0,$A0,$C2,$B0,$C6,$C6,$A0,$A0
	.byte	$BB,$A0,$A0,$A0,$A0,$A0,$A3,$A0,$A0,$A0,$A0,$FF,$A0,$A0
	.byte	$A0,$A0,$BA,$B3,$C6,$CD,$A0,$A0,$A0,$A0,$A0,$BA,$A0,$AC
	.byte	$B4,$A0,$A0,$B5,$A0,$A0,$A0,$B1,$A0,$B4,$A0,$A0,$A0,$80
	.byte	$A0,$C6,$B2,$BB,$BF,$A0,$FF,$A0,$C2,$B1,$A0,$A0,$A0,$A0
	.byte	$C4,$A0,$A0,$C2,$A0,$C6,$C6,$A0,$A0,$A0,$A0,$A0,$A0,$81
	.byte	$A0,$E0,$D0,$A0,$A0,$D4,$FF,$A0,$AE,$A0,$A0,$BA,$A0,$B2
	.byte	$A0,$A0,$A0,$A0,$A0,$A0,$BA,$A0,$A0,$B4,$A0,$A0,$B5,$A0
	.byte	$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$80,$A0,$A0,$B2,$BB,$80
	.byte	$D2,$FF,$A0,$C2,$A0,$A0,$B1,$A0,$AC,$C4,$C1,$A0,$A0,$A0
	.byte	$A0,$C6,$A0,$C5,$A0,$A0,$A0,$A0,$81,$A0,$E0,$A0,$A0,$A0
	.byte	$D4,$BF,$A0,$AE,$A0,$A0,$BA,$A0,$B2,$A0,$CC,$A0,$A0,$A0
;LAB_FFF1:
	.byte	$A0,$BA,$D7,$A0,$B4,$D9,$A0,$B6,$B0


;************************************************************************************
;
; hardware vectors

;	.ORG	$FFFA			; vectors start

;LAB_FFFA:
	.word	LAB_F5AC		; NMI vector, triggered by the VIA T1
	.word	LAB_F03C		; RESET vector
	.word	LAB_F499		; IRQ vector, triggered by the ACIAs

	.END


;************************************************************************************
